@@ -1,6 +1,6 @@
 /***************************************************************************
  * fltk_example.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -125,20 +125,22 @@ int main(int argc,char **argv)
 	else	printf("You may specify argument '1', '2', '3', 'd' for viewing examples of 1d, 2d, 3d, dual plotting,\nor 'm' for multi-threading sample.\n");
 	switch(key)
 	{
-	case '0':	gr = new mglFLTK((mglDraw *)NULL,"1D plots");	break;
+	case '0':	gr = new mglFLTK((mglDraw *)NULL,"1D plots");
+				gr->Rotate(40,60);	gr->Box();	gr->Light(true);
+				gr->FSurf("sin(4*pi*x*y)");	gr->Update();	break;
 	case '1':	gr = new mglFLTK(sample_1,"1D plots");	break;
 	case '2':	gr = new mglFLTK(sample_2,"2D plots");	break;
 	case '3':	gr = new mglFLTK(sample_3,"3D plots");	break;
 	case 'd':	gr = new mglFLTK(sample_d,"Dual plots");break;
 	case 't':	gr = new mglFLTK(test_wnd,"Testing");	break;
+	case 'f':	gr = new mglFLTK("Frame drawing");
+				gr->NewFrame();	gr->Box();	gr->EndFrame();	break;
 #if MGL_HAVE_PTHR_WIDGET
 	case 'm':	gr = new mglFLTK(&dr,"Multi-threading test");
 	dr.SetWnd(gr);	dr.Run();	break;
 #endif
 	default:	gr = new mglFLTK(sample,"Drop and waves");	break;
 	}
-	if(key=='0')
-	{	gr->Rotate(40,60);	gr->Box();	gr->Light(true);	gr->FSurf("sin(4*pi*x*y)");	gr->Update();	}
 	gr->Run();	return 0;
 }
 #endif
