@@ -545,6 +545,13 @@ inline mglData mglIFS3d(const mglDataA &A, long n, long skip=20)
 /// Get array which is n-th points {x[i],y[i],z[i]} for iterated function system (fractal) defined in *.ifs file 'fname' and named as 'name'
 inline mglData mglIFSfile(const char *fname, const char *name, long n, long skip=20)
 {	return mglData(true,mgl_data_ifs_file(fname,name,n,skip));	}
+/// Get array which is n-th pairs {x[i],y[i]} for flame fractal generated by A with functions F
+/// Get array which is n-th pairs {x[i],y[i]} for Flame fractal generated by A with functions F
+/** NOTE: A.nx must be >= 7 and F.nx >= 2 and F.nz=A.ny.
+ * F[0,i,j] denote function id. F[1,i,j] give function weight, F(2:5,i,j) provide function parameters.
+ * Resulting point is {xnew,ynew} = sum_i F[1,i,j]*F[0,i,j]{IFS2d(A[j]){x,y}}. */
+inline mglData mglFlame2d(const mglDataA &A, const mglDataA &F, long n, long skip=20)
+{	return mglData(true,mgl_data_flame_2d(&A,&F,n,skip));	}
 //-----------------------------------------------------------------------------
 /// Get sub-array of the data with given fixed indexes
 inline mglData mglSubData(const mglDataA &dat, long xx, long yy=-1, long zz=-1)
