@@ -207,6 +207,26 @@ uintptr_t MGL_EXPORT mgl_data_ifs_3d_(uintptr_t *A, long *n, long *skip);
 /// Get array which is n-th points {x[i],y[i],z[i]} for iterated function system (fractal) defined in *.ifs file 'fname' and named as 'name'
 HMDT MGL_EXPORT mgl_data_ifs_file(const char *fname, const char *name, long n, long skip);
 uintptr_t mgl_data_ifs_file_(const char *fname, const char *name, long *n, long *skip,int l,int m);
+/// Codes for flame fractal functions
+enum {
+	mglFlameLinear=0,		mglFlameSin,			mglFlameSphere,		mglFlameSwirl,		mglFlameHorseshoe,
+	mglFlamePolar,			mglFlameHandkerchief,	mglFlameHeart,		mglFlameDisk,		mglFlameSpiral,
+	mglFlameHyperbolic,		mglFlameDiamond,		mglFlameEx,			mglFlameJulia,		mglFlameBent,
+	mglFlameWaves,			mglFlameFishEye,		mglFlamePopcorn,	mglFlameExponent,	mglFlamePower,
+	mglFlameCos,			mglFlameRings,			mglFlameFan,		mglFlameBlob,		mglFlamePdj,
+	mglFlameFan2,			mglFlameRings2,			mglFlameEyefish,	mglFlameBubble,		mglFlameCylinder,
+	mglFlamePerspective,	mglFlameNoise,			mglFlameJuliaN,		mglFlameJuliaScope,	mglFlameBlur,
+	mglFlameGaussian,		mglFlameRadialBlur,		mglFlamePie,		mglFlameNgon,		mglFlameCurl,
+	mglFlameRectangles,		mglFlameArch,			mglFlameTangent,	mglFlameSquare,		mglFlameRays,
+	mglFlameBlade,			mglFlameSecant,			mglFlameTwintrian,	mglFlameCross,		mglFlameLAST
+};
+/// Get array which is n-th pairs {x[i],y[i]} for Flame fractal generated by A with functions F
+/** NOTE: A.nx must be >= 7 and F.nx >= 2 and F.nz=A.ny.
+ * F[0,i,j] denote function id. F[1,i,j] give function weight. F(2:5,i,j) provide function parameters.
+ * Resulting point is {xnew,ynew} = sum_i F[1,i,j]*F[0,i,j]{IFS2d(A[j]){x,y}}. */
+HMDT MGL_EXPORT mgl_data_flame_2d(HCDT A, HCDT F, long n, long skip);
+uintptr_t MGL_EXPORT mgl_data_flame_2d_(uintptr_t *A, uintptr_t *F, long *n, long *skip);
+
 
 /// Get array as solution of tridiagonal matrix solution a[i]*x[i-1]+b[i]*x[i]+c[i]*x[i+1]=d[i]
 /** String \a how may contain:
