@@ -1,6 +1,6 @@
 /***************************************************************************
  * data_cf.h is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -88,6 +88,16 @@ mdual MGL_EXPORT mgl_datac_get_value_(uintptr_t *d, int *i, int *j, int *k);
 void MGL_EXPORT mgl_datac_set_values(HADT dat, const char *val, long nx, long ny, long nz);
 void MGL_EXPORT mgl_datac_set_values_(uintptr_t *d, const char *val, int *nx, int *ny, int *nz, int l);
 
+/// Get array as solution of tridiagonal matrix solution a[i]*x[i-1]+b[i]*x[i]+c[i]*x[i+1]=d[i]
+/** String \a how may contain:
+ * 'x', 'y', 'z' for solving along x-,y-,z-directions, or
+ * 'h' for solving along hexagonal direction at x-y plain (need nx=ny),
+ * 'c' for using periodical boundary conditions,
+ * 'd' for diffraction/diffuse calculation.
+ * NOTE: It work for flat data model only (i.e. for a[i,j]==a[i+nx*j]) */
+HADT MGL_EXPORT mgl_datac_tridmat(HCDT A, HCDT B, HCDT C, HCDT D, const char *how);
+uintptr_t MGL_EXPORT mgl_datac_tridmat_(uintptr_t *A, uintptr_t *B, uintptr_t *C, uintptr_t *D, const char *how, int);
+
 /// Returns pointer to internal data array
 MGL_EXPORT dual *mgl_datac_data(HADT dat);
 /// Returns pointer to data element [i,j,k]
@@ -196,6 +206,10 @@ void MGL_EXPORT mgl_datac_modify_(uintptr_t *dat, const char *eq,int *dim,int);
 void MGL_EXPORT mgl_datac_modify_vw(HADT dat, const char *eq,HCDT vdat,HCDT wdat);
 void MGL_EXPORT mgl_datac_modify_vw_(uintptr_t *dat, const char *eq, uintptr_t *vdat, uintptr_t *wdat,int);
 
+/// Limit the data to be inside [-v,v], keeping the original sign
+void MGL_EXPORT mgl_datac_limit(HADT dat, mreal v);
+void MGL_EXPORT mgl_datac_limit_(uintptr_t *dat, mreal *v);
+
 /// Put value to data element(s)
 void MGL_EXPORT mgl_datac_put_val(HADT dat, dual val, long i, long j, long k);
 void MGL_EXPORT mgl_datac_put_val_(uintptr_t *dat, dual *val, int *i, int *j, int *k);
@@ -291,14 +305,21 @@ uintptr_t MGL_EXPORT mgl_datac_correl_(uintptr_t *dat1, uintptr_t *dat2, const c
 void MGL_EXPORT mgl_datac_diffr(HADT dat, const char *how, mreal q);
 void MGL_EXPORT mgl_datac_diffr_(uintptr_t *d, const char *how, double q,int l);
 
+/// Get real part of data values
 HMDT MGL_EXPORT mgl_datac_real(HCDT dat);
 uintptr_t MGL_EXPORT mgl_datac_real_(uintptr_t *dat);
+/// Get imaginary part of data values
 HMDT MGL_EXPORT mgl_datac_imag(HCDT dat);
 uintptr_t MGL_EXPORT mgl_datac_imag_(uintptr_t *dat);
+/// Get absolute value of data values, i.e. |u|
 HMDT MGL_EXPORT mgl_datac_abs(HCDT dat);
 uintptr_t MGL_EXPORT mgl_datac_abs_(uintptr_t *dat);
+/// Get argument of data values
 HMDT MGL_EXPORT mgl_datac_arg(HCDT dat);
 uintptr_t MGL_EXPORT mgl_datac_arg_(uintptr_t *dat);
+/// Get square of absolute value of data values, i.e. |u|^2
+HMDT MGL_EXPORT mgl_datac_norm(HCDT dat);
+uintptr_t MGL_EXPORT mgl_datac_norm_(uintptr_t *dat);
 
 /// Interpolate by linear function the data to given point x=[0...nx-1], y=[0...ny-1], z=[0...nz-1]
 mdual MGL_EXPORT mgl_datac_linear(HCDT d, mreal x,mreal y,mreal z);
