@@ -330,20 +330,21 @@ HADT MGL_EXPORT mgl_datac_sum(HCDT dat, const char *dir)
 	if(strchr(dir,'z') && nz>1)
 	{
 		mglStartThreadC(mgl_sumc_z,0,nx*ny,b,c,0,p);
-		memcpy(c,b,nx*ny*sizeof(mreal));	p[2] = 1;
+		memcpy(c,b,nx*ny*sizeof(dual));	p[2] = 1;
 	}
 	if(strchr(dir,'y') && ny>1)
 	{
 		mglStartThreadC(mgl_sumc_y,0,nx*p[2],b,c,0,p);
-		memcpy(c,b,nx*p[2]*sizeof(mreal));	p[1] = p[2];	p[2] = 1;
+		memcpy(c,b,nx*p[2]*sizeof(dual));	p[1] = p[2];	p[2] = 1;
 	}
 	if(strchr(dir,'x') && nx>1)
 	{
 		mglStartThreadC(mgl_sumc_x,0,p[1]*p[2],b,c,0,p);
 		p[0] = p[1];	p[1] = p[2];	p[2] = 1;
+		memcpy(c,b,p[0]*p[1]*sizeof(dual));
 	}
 	mglDataC *r=new mglDataC(p[0],p[1],p[2]);
-	memcpy(r->a,b,p[0]*p[1]*p[2]*sizeof(dual));
+	memcpy(r->a,c,p[0]*p[1]*p[2]*sizeof(dual));
 	delete []b;	delete []c;	return r;
 }
 uintptr_t MGL_EXPORT mgl_datac_sum_(uintptr_t *d, const char *dir,int l)
