@@ -1,6 +1,6 @@
 /***************************************************************************
  * evalc.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -30,6 +30,9 @@ EQ_NUM=0,	// a variable substitution
 EQ_RND,		// random number
 EQ_A,		// numeric constant
 // normal functions of 2 arguments
+EQ_LT,		// comparison x<y			!!! MUST BE FIRST 2-PLACE FUNCTION
+EQ_GT,		// comparison x>y
+EQ_EQ,		// comparison x=y
 EQ_ADD,		// addition x+y
 EQ_SUB,		// substraction x-y
 EQ_MUL,		// multiplication x*y
@@ -58,6 +61,9 @@ EQ_LG,		// decimal logarithm of x, lg(x) = ln(x)/ln(10)
 EQ_ABS,		// absolute value
 EQ_ARG,		// argument (or phase) of complex number
 EQ_CONJ,	// complex conjugate
+EQ_REAL,	// real part
+EQ_IMAG,	// imaginary part
+EQ_NORM,	// square of absolute value |u|^2
 EQ_LAST		// id of last entry
 };
 //-----------------------------------------------------------------------------
@@ -91,6 +97,17 @@ mglFormulaC::mglFormulaC(const char *string)
 		len-=2;	str[len]=0;
 	}
 	len=strlen(str);
+	n=mglFindInText(str,"<>=");				// low priority -- conditions
+	if(n>=0)
+	{
+		if(str[n]=='<') Kod=EQ_LT;
+		else if(str[n]=='>') Kod=EQ_GT;
+		else Kod=EQ_EQ;
+		str[n]=0;
+		Left=new mglFormulaC(str);
+		Right=new mglFormulaC(str+n+1);
+		delete []str;	return;
+	}
 	n=mglFindInText(str,"+-");				// normal priority -- additions
 	if(n>=0 && (n<2 || str[n-1]!='e' || (str[n-2]!='.' && !isdigit(str[n-2]))))
 	{
@@ -159,6 +176,9 @@ mglFormulaC::mglFormulaC(const char *string)
 		else if(!strcmp(name,"abs")) Kod=EQ_ABS;
 		else if(!strcmp(name,"arg")) Kod=EQ_ARG;
 		else if(!strcmp(name,"conj")) Kod=EQ_CONJ;
+		else if(!strcmp(name,"real")) Kod=EQ_REAL;
+		else if(!strcmp(name,"imag")) Kod=EQ_IMAG;
+		else if(!strcmp(name,"norm")) Kod=EQ_NORM;
 		else {	delete []str;	return;	}	// unknown function
 		n=mglFindInText(str,",");
 		if(n>=0)
@@ -208,6 +228,9 @@ dual mglFormulaC::Calc(const dual var[MGL_VS]) const
 	return CalcIn(var);
 }
 //-----------------------------------------------------------------------------
+dual MGL_LOCAL_CONST ceqc(dual a,dual b)	{return a==b?1:0;}
+dual MGL_LOCAL_CONST cltc(dual a,dual b)	{return real(a-b)<0?1:0;}
+dual MGL_LOCAL_CONST cgtc(dual a,dual b)	{return real(a-b)>0?1:0;}
 dual MGL_LOCAL_CONST addc(dual a,dual b)	{return a+b;}
 dual MGL_LOCAL_CONST subc(dual a,dual b)	{return a-b;}
 dual MGL_LOCAL_CONST mulc(dual a,dual b)	{return a*b;}
@@ -238,15 +261,18 @@ dual MGL_LOCAL_CONST logc(dual x)	{	return log(x);	}
 dual MGL_LOCAL_CONST absc(dual x)	{	return abs(x);	}
 dual MGL_LOCAL_CONST argc(dual x)	{	return arg(x);	}
 dual MGL_LOCAL_CONST lgc(dual x)	{	return log10(x);}
+dual MGL_LOCAL_CONST realc(dual x)	{	return real(x);	}
+dual MGL_LOCAL_CONST imagc(dual x)	{	return imag(x);	}
+dual MGL_LOCAL_CONST normc(dual x)	{	return norm(x);	}
 //-----------------------------------------------------------------------------
 typedef dual (*func_1)(dual);
 typedef dual (*func_2)(dual, dual);
+static const func_2 f2[EQ_SIN-EQ_LT] = {cltc,cgtc,ceqc,addc,subc,mulc,divc,ipwc,powc,llgc};
+static const func_1 f1[EQ_LAST-EQ_SIN] = {sinc,cosc,tanc,asinc,acosc,atanc,sinhc,coshc,tanhc,
+					asinhc,acoshc,atanhc,sqrtc,expc,expi,logc,lgc,absc,argc,conjc,realc,imagc,normc};
 // evaluation of embedded (included) expressions
 dual mglFormulaC::CalcIn(const dual *a1) const
 {
-	func_2 f2[EQ_SIN-EQ_ADD] = {addc,subc,mulc,divc,ipwc,powc,llgc};
-	func_1 f1[EQ_LAST-EQ_SIN] = {sinc,cosc,tanc,asinc,acosc,atanc,sinhc,coshc,tanhc,
-					asinhc,acoshc,atanhc,sqrtc,expc,expi,logc,lgc,absc,argc,conjc};
 //	if(Error)	return 0;
 	if(Kod==EQ_A)	return a1[(int)Res.real()];
 	if(Kod==EQ_RND)	return mgl_rnd();
@@ -258,7 +284,7 @@ dual mglFormulaC::CalcIn(const dual *a1) const
 		if(Kod<EQ_SIN)
 		{
 			dual b = Right->CalcIn(a1);
-			b = mgl_isfin(b)?f2[Kod-EQ_ADD](a,b):NAN;
+			b = mgl_isfin(b)?f2[Kod-EQ_LT](a,b):NAN;
 			return mgl_isfin(b)?b:NAN;
 		}
 		else
