@@ -1,6 +1,6 @@
 /***************************************************************************
  * evalp.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -143,13 +143,16 @@ HMDT mglApplyOperDiv(std::wstring a1, std::wstring a2, mglParser *arg, const std
 	mreal va=a->a[0], vb=b->a[0], *aa=a->a, *bb=b->a, *cc=r->a;
 	if(na==nb)
 #pragma omp parallel for
-		for(long i=0;i<nn;i++)	cc[i] = aa[i]/bb[i];
+		for(long i=0;i<nn;i++)	cc[i] = bb[i]!=0?aa[i]/bb[i]:NAN;
 	else if(na==1)
 #pragma omp parallel for
-		for(long i=0;i<nn;i++)	cc[i] = va/bb[i];
-	else
+		for(long i=0;i<nn;i++)	cc[i] = bb[i]!=0?va/bb[i]:NAN;
+	else if(vb!=0)
 #pragma omp parallel for
 		for(long i=0;i<nn;i++)	cc[i] = aa[i]/vb;
+	else
+#pragma omp parallel for
+		for(long i=0;i<nn;i++)	cc[i] = NAN;
 	mgl_delete_data(d);	return r;
 }
 //-----------------------------------------------------------------------------
@@ -241,13 +244,16 @@ HADT mglApplyOperDivC(std::wstring a1, std::wstring a2, mglParser *arg, const st
 	dual va=a->a[0], vb=b->a[0], *aa=a->a, *bb=b->a, *cc=r->a;
 	if(na==nb)
 #pragma omp parallel for
-		for(long i=0;i<nn;i++)	cc[i] = aa[i]/bb[i];
+		for(long i=0;i<nn;i++)	cc[i] = bb[i]!=mreal(0)?aa[i]/bb[i]:NAN;
 	else if(na==1)
 #pragma omp parallel for
-		for(long i=0;i<nn;i++)	cc[i] = va/bb[i];
-	else
+		for(long i=0;i<nn;i++)	cc[i] = bb[i]!=mreal(0)?va/bb[i]:NAN;
+	else if(vb!=mreal(0))
 #pragma omp parallel for
 		for(long i=0;i<nn;i++)	cc[i] = aa[i]/vb;
+	else
+#pragma omp parallel for
+		for(long i=0;i<nn;i++)	cc[i] = NAN;
 	mgl_delete_datac(d);	return r;
 }
 //-----------------------------------------------------------------------------
@@ -488,7 +494,7 @@ HMDT MGL_NO_EXPORT mglFormulaCalc(std::wstring str, mglParser *arg, const std::v
 		else
 		{
 			HMDT res = new mglData;
-			char ch = str[0];
+			wchar_t ch = str[0];
 			if(ch<':') res->a[0] = wcstod(str.c_str(),0);	// this is number
 			else if(!str.compare(L"pi"))	res->a[0] = M_PI;
 			else if(ch==':')	res->a[0] = -1;
@@ -686,6 +692,12 @@ HMDT MGL_NO_EXPORT mglFormulaCalc(std::wstring str, mglParser *arg, const std::v
 			HMDT res = mgl_datac_imag(a1);
 			mgl_delete_datac(a1);	return res;
 		}
+		else if(!nm.compare(L"norm"))
+		{
+			HADT a1 = mglFormulaCalcC(str, arg, head);
+			HMDT res = mgl_datac_norm(a1);
+			mgl_delete_datac(a1);	return res;
+		}
 #if MGL_HAVE_GSL
 		else if(!nm.compare(L"i") && n>0)
 			return mglApplyOper(str.substr(0,n),str.substr(n+1),arg, head, gsl_sf_bessel_Inu);
@@ -739,9 +751,10 @@ dual MGL_LOCAL_CONST sqrtc(dual x);	//{	return sqrt(x);	}
 dual MGL_LOCAL_CONST logc(dual x);	//{	return log(x);	}
 dual MGL_LOCAL_CONST absc(dual x);	//{	return abs(x);	}
 dual MGL_LOCAL_CONST argc(dual x);	//{	return arg(x);	}
-dual MGL_LOCAL_CONST lgc(dual x);		//{	return log10(x);}
-dual MGL_LOCAL_CONST realc(dual x)	{	return real(x);	}
-dual MGL_LOCAL_CONST imagc(dual x)	{	return dual(0,imag(x));	}
+dual MGL_LOCAL_CONST lgc(dual x);	//{	return log10(x);}
+dual MGL_LOCAL_CONST realc(dual x);	//{	return real(x);	}
+dual MGL_LOCAL_CONST imagc(dual x);	//{	return imag(x);	}
+dual MGL_LOCAL_CONST normc(dual x);	//{	return norm(x);	}
 //-----------------------------------------------------------------------------
 /// Parse string and substitute the script argument
 // All numbers are presented as mglData(1). Do boundary checking.
@@ -907,7 +920,7 @@ HADT MGL_NO_EXPORT mglFormulaCalcC(std::wstring str, mglParser *arg, const std::
 		else
 		{
 			HADT res = new mglDataC;
-			char ch = str[0];
+			wchar_t ch = str[0];
 			if(ch<':')	// this is real number
 				res->a[0] = (str[str.length()-1]=='i') ? dual(0,wcstod(str.c_str(),0)) :  mreal(wcstod(str.c_str(),0));
 			else if(ch=='i')	// this is imaginary number
@@ -970,7 +983,7 @@ HADT MGL_NO_EXPORT mglFormulaCalcC(std::wstring str, mglParser *arg, const std::
 			else if(!nm.compare(L"acosh"))	return mglApplyFuncC(str, arg, head, acoshc);
 			else if(!nm.compare(L"atanh"))	return mglApplyFuncC(str, arg, head, atanhc);
 			else if(!nm.compare(L"arg"))	return mglApplyFuncC(str, arg, head, argc);
-			else if(!nm.compare(L"abs"))		return mglApplyFuncC(str, arg, head, absc);
+			else if(!nm.compare(L"abs"))	return mglApplyFuncC(str, arg, head, absc);
 		}
 		else if(nm[0]=='c')
 		{
@@ -1002,6 +1015,7 @@ HADT MGL_NO_EXPORT mglFormulaCalcC(std::wstring str, mglParser *arg, const std::
 			for(long i=0;i<res->GetNN();i++)	a[i] = dual(mgl_rnd(), mgl_rnd());	return res;	}
 		else if(!nm.compare(L"real"))	return mglApplyFuncC(str, arg, head, realc);
 		else if(!nm.compare(L"imag"))	return mglApplyFuncC(str, arg, head, imagc);
+		else if(!nm.compare(L"norm"))	return mglApplyFuncC(str, arg, head, normc);
 	}
 	HADT res = new mglDataC;	res->a[0]=NAN;	return res;
 }
