@@ -1,6 +1,6 @@
 /***************************************************************************
  * fft.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -1072,12 +1072,12 @@ void MGL_EXPORT mgl_data_hankel_(uintptr_t *d, const char *dir,int l)
 void MGL_EXPORT mgl_data_fill_sample(HMDT d, const char *how)
 {
 	if(!how || *how==0)	return;
-	bool kk = strchr(how,'k');
+	bool kk = mglchr(how,'k');
 	long n=d->nx,dn=1;
 	mreal *aa=d->a;
-	if(strchr(how,'y'))	{	n=d->ny;	dn=d->nx;	}
-	if(strchr(how,'z'))	{	n=d->nz;	dn=d->nx*d->ny;	}
-	if(strchr(how,'h'))	// Hankel
+	if(mglchr(how,'y'))	{	n=d->ny;	dn=d->nx;	}
+	if(mglchr(how,'z'))	{	n=d->nz;	dn=d->nx*d->ny;	}
+	if(mglchr(how,'h'))	// Hankel
 	{
 #if MGL_HAVE_GSL
 		gsl_dht *dht = gsl_dht_new(n,0,1);
