@@ -1,6 +1,6 @@
 /***************************************************************************
  * other.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -24,19 +24,9 @@
 #include "mgl2/data.h"
 #include "mgl2/base.h"
 //-----------------------------------------------------------------------------
-//
-//	DensX, DensY, DensZ series
-//
-//-----------------------------------------------------------------------------
-void MGL_EXPORT mgl_dens_x(HMGL gr, HCDT a, const char *sch, double sv, const char *opt)
+HCDT MGL_NO_EXPORT fill_slice_x(HMGL gr, double sv, HCDT a, mglDataV &xx, mglDataV &yy, mglDataV &zz, mglData &aa)
 {
 	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
-	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
-	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"DensX");	return;	}
-	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"DensX");	return;	}
-	mglData xx,yy,zz,aa;
-	gr->SaveState(opt);
-
 	if(l>1)
 	{
 		aa.Create(m,l);	xx.Create(m,l);	yy.Create(m,l);	zz.Create(m,l);
@@ -44,15 +34,9 @@ void MGL_EXPORT mgl_dens_x(HMGL gr, HCDT a, const char *sch, double sv, const ch
 		long k = long(d);	d = d - k;
 		if(k>n-2)	{	k=n-2;	d=1;	}
 		if(k<0)		{	k=0;	d=0;	}
-		const mglData *ma=dynamic_cast<const mglData *>(a);
-		if(ma)
 #pragma omp parallel for collapse(2)
-			for(long j=0;j<l;j++)	for(long i=0;i<m;i++)
-				aa.a[i+m*j] = ma->a[k+n*(i+m*j)]*(1-d) + d*ma->a[k+1+n*(i+m*j)];
-		else
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<l;j++)	for(long i=0;i<m;i++)
-				aa.a[i+m*j] = a->v(k,i,j)*(1-d) + d*a->v(k+1,i,j);
+		for(long j=0;j<l;j++)	for(long i=0;i<m;i++)
+			aa.a[i+m*j] = a->v(k,i,j)*(1-d) + d*a->v(k+1,i,j);
 		a = &aa;
 	}
 	else
@@ -60,18 +44,12 @@ void MGL_EXPORT mgl_dens_x(HMGL gr, HCDT a, const char *sch, double sv, const ch
 	xx.Fill(sv, sv);
 	yy.Fill(gr->Min.y, gr->Max.y,'x');
 	zz.Fill(gr->Min.z, gr->Max.z,'y');
-	mgl_surfc_xy(gr,&xx,&yy,&zz,a,sch,0);
+	return a;
 }
 //-----------------------------------------------------------------------------
-void MGL_EXPORT mgl_dens_y(HMGL gr, HCDT a, const char *sch, double sv, const char *opt)
+HCDT MGL_NO_EXPORT fill_slice_y(HMGL gr, double sv, HCDT a, mglDataV &xx, mglDataV &yy, mglDataV &zz, mglData &aa)
 {
 	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
-	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
-	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"DensY");	return;	}
-	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"DensY");	return;	}
-	mglData xx,yy,zz,aa;
-	gr->SaveState(opt);
-
 	if(l>1)
 	{
 		aa.Create(n,l);	xx.Create(n,l);	yy.Create(n,l);	zz.Create(n,l);
@@ -79,15 +57,9 @@ void MGL_EXPORT mgl_dens_y(HMGL gr, HCDT a, const char *sch, double sv, const ch
 		long k = long(d);	d = d - k;
 		if(k>m-2)	{	k=m-2;	d=1;	}
 		if(k<0)		{	k=0;	d=0;	}
-		const mglData *ma=dynamic_cast<const mglData *>(a);
-		if(ma)
 #pragma omp parallel for collapse(2)
-			for(long j=0;j<l;j++)	for(long i=0;i<n;i++)
-				aa.a[i+n*j] = ma->a[i+n*(k+m*j)]*(1-d) + d*ma->a[i+n+n*(k+m*j)];
-		else
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<l;j++)	for(long i=0;i<n;i++)
-				aa.a[i+n*j] = a->v(i,k,j)*(1-d) + d*a->v(i,k+1,j);
+		for(long j=0;j<l;j++)	for(long i=0;i<n;i++)
+			aa.a[i+n*j] = a->v(i,k,j)*(1-d) + d*a->v(i,k+1,j);
 		a = &aa;
 	}
 	else
@@ -95,18 +67,12 @@ void MGL_EXPORT mgl_dens_y(HMGL gr, HCDT a, const char *sch, double sv, const ch
 	yy.Fill(sv, sv);
 	xx.Fill(gr->Min.x, gr->Max.x,'x');
 	zz.Fill(gr->Min.z, gr->Max.z,'y');
-	mgl_surfc_xy(gr,&xx,&yy,&zz,a,sch,0);
+	return a;
 }
 //-----------------------------------------------------------------------------
-void MGL_EXPORT mgl_dens_z(HMGL gr, HCDT a, const char *sch, double sv, const char *opt)
+HCDT MGL_NO_EXPORT fill_slice_z(HMGL gr, double sv, HCDT a, mglDataV &xx, mglDataV &yy, mglDataV &zz, mglData &aa)
 {
 	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
-	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
-	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"DensZ");	return;	}
-	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"DensZ");	return;	}
-	mglData xx,yy,zz,aa;
-	gr->SaveState(opt);
-
 	xx.Create(n,m);	yy.Create(n,m);	zz.Create(n,m);
 	if(l>1)
 	{
@@ -115,20 +81,54 @@ void MGL_EXPORT mgl_dens_z(HMGL gr, HCDT a, const char *sch, double sv, const ch
 		long k = long(d);	d = d - k;
 		if(k>l-2)	{	k=l-2;	d=1;	}
 		if(k<0)		{	k=0;	d=0;	}
-		const mglData *ma=dynamic_cast<const mglData *>(a);
-		if(ma)
 #pragma omp parallel for collapse(2)
-			for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
-				aa.a[i+n*j] = ma->a[i+n*(j+m*k)]*(1-d) + d*ma->a[i+n*m+n*(j+m*k)];
-		else
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
-				aa.a[i+n*j] = a->v(i,j,k)*(1-d) + d*a->v(i,j,k+1);
+		for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
+			aa.a[i+n*j] = a->v(i,j,k)*(1-d) + d*a->v(i,j,k+1);
 		a = &aa;
 	}
 	zz.Fill(sv, sv);
 	yy.Fill(gr->Min.y, gr->Max.y,'y');
 	xx.Fill(gr->Min.x, gr->Max.x,'x');
+	return a;
+}
+//-----------------------------------------------------------------------------
+//
+//	DensX, DensY, DensZ series
+//
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_dens_x(HMGL gr, HCDT a, const char *sch, double sv, const char *opt)
+{
+	long n=a->GetNx(),m=a->GetNy();
+	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"DensX");	return;	}
+	gr->SaveState(opt);
+	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
+	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"DensX");	gr->LoadState();	return;	}
+	mglDataV xx,yy,zz;	mglData aa;
+	a = fill_slice_x(gr,sv,a,xx,yy,zz,aa);
+	mgl_surfc_xy(gr,&xx,&yy,&zz,a,sch,0);
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_dens_y(HMGL gr, HCDT a, const char *sch, double sv, const char *opt)
+{
+	long n=a->GetNx(),m=a->GetNy();
+	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"DensY");	return;	}
+	gr->SaveState(opt);
+	if(mgl_isnan(sv))	sv = gr->GetOrgY('y');
+	if(sv<gr->Min.y || sv>gr->Max.y)	{	gr->SetWarn(mglWarnSlc,"DensY");	gr->LoadState();	return;	}
+	mglDataV xx,yy,zz;	mglData aa;
+	a = fill_slice_y(gr,sv,a,xx,yy,zz,aa);
+	mgl_surfc_xy(gr,&xx,&yy,&zz,a,sch,0);
+}
+//-----------------------------------------------------------------------------
+void MGL_EXPORT mgl_dens_z(HMGL gr, HCDT a, const char *sch, double sv, const char *opt)
+{
+	long n=a->GetNx(),m=a->GetNy();
+	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"DensZ");	return;	}
+	gr->SaveState(opt);
+	if(mgl_isnan(sv))	sv = gr->GetOrgZ('z');
+	if(sv<gr->Min.z || sv>gr->Max.z)	{	gr->SetWarn(mglWarnSlc,"DensZ");	gr->LoadState();	return;	}
+	mglDataV xx,yy,zz;	mglData aa;
+	a = fill_slice_z(gr,sv,a,xx,yy,zz,aa);
 	mgl_surfc_xy(gr,&xx,&yy,&zz,a,sch,0);
 }
 //-----------------------------------------------------------------------------
@@ -156,41 +156,21 @@ void MGL_EXPORT mgl_dens_z_(uintptr_t *gr, uintptr_t *a, const char *sch, mreal
 void MGL_EXPORT mgl_cont_gen(HMGL gr, mreal val, HCDT a, HCDT x, HCDT y, HCDT z, mreal c, int text,long ak);
 void MGL_EXPORT mgl_cont_x_val(HMGL gr, HCDT v, HCDT a, const char *sch, double sv, const char *opt)
 {
-	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
-	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
+	long n=a->GetNx(),m=a->GetNy();
 	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"ContX");	return;	}
-	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"ContX");	return;	}
 	gr->SaveState(opt);
+	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
+	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"ContX");	gr->LoadState();	return;	}
 	static int cgid=1;	gr->StartGroup("ContX",cgid++);
-	mglData xx,yy,zz,aa;
+	mglDataV xx,yy,zz;	mglData aa;
 
-	bool text=(mglchr(sch,'t'));
+	int text=0;
+	if(mglchr(sch,'t'))	text=1;
+	if(mglchr(sch,'T'))	text=2;
 	long ss=gr->AddTexture(sch);
 	gr->SetPenPal(sch);
 
-	if(l>1)
-	{
-		aa.Create(m,l);	xx.Create(m,l);	yy.Create(m,l);	zz.Create(m,l);
-		mreal d = (n-1)*(sv - gr->Min.x)/(gr->Max.x - gr->Min.x);
-		long k = long(d);	d = d - k;
-		if(k>n-2)	{	k=n-2;	d=1;	}
-		if(k<0)		{	k=0;	d=0;	}
-		const mglData *ma=dynamic_cast<const mglData *>(a);
-		if(ma)
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<l;j++)	for(long i=0;i<m;i++)
-				aa.a[i+m*j] = ma->a[k+n*(i+m*j)]*(1-d) + d*ma->a[k+1+n*(i+m*j)];
-		else
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<l;j++)	for(long i=0;i<m;i++)
-				aa.a[i+m*j] = a->v(k,i,j)*(1-d) + d*a->v(k+1,i,j);
-		a = &aa;
-	}
-	else
-	{	xx.Create(n,m);	yy.Create(n,m);	zz.Create(n,m);	}
-	xx.Fill(sv, sv);
-	yy.Fill(gr->Min.y, gr->Max.y,'x');
-	zz.Fill(gr->Min.z, gr->Max.z,'y');
+	a = fill_slice_x(gr,sv,a,xx,yy,zz,aa);
 #pragma omp parallel for
 	for(long i=0;i<v->GetNx();i++)
 	{
@@ -202,41 +182,21 @@ void MGL_EXPORT mgl_cont_x_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cont_y_val(HMGL gr, HCDT v, HCDT a, const char *sch, double sv, const char *opt)
 {
-	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
-	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
+	long n=a->GetNx(),m=a->GetNy();
 	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"ContY");	return;	}
-	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"ContY");	return;	}
 	gr->SaveState(opt);
+	if(mgl_isnan(sv))	sv = gr->GetOrgY('y');
+	if(sv<gr->Min.y || sv>gr->Max.y)	{	gr->SetWarn(mglWarnSlc,"ContY");	gr->LoadState();	return;	}
 	static int cgid=1;	gr->StartGroup("ContY",cgid++);
-	mglData xx,yy,zz,aa;
+	mglDataV xx,yy,zz;	mglData aa;
 
-	bool text=(mglchr(sch,'t'));
+	int text=0;
+	if(mglchr(sch,'t'))	text=1;
+	if(mglchr(sch,'T'))	text=2;
 	long ss=gr->AddTexture(sch);
 	gr->SetPenPal(sch);
 
-	if(l>1)
-	{
-		aa.Create(n,l);	xx.Create(n,l);	yy.Create(n,l);	zz.Create(n,l);
-		mreal d = (m-1)*(sv - gr->Min.y)/(gr->Max.y - gr->Min.y);
-		long k = long(d);	d = d - k;
-		if(k>m-2)	{	k=m-2;	d=1;	}
-		if(k<0)		{	k=0;	d=0;	}
-		const mglData *ma=dynamic_cast<const mglData *>(a);
-		if(ma)
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<l;j++)	for(long i=0;i<n;i++)
-				aa.a[i+n*j] = ma->a[i+n*(k+m*j)]*(1-d) + d*ma->a[i+n+n*(k+m*j)];
-		else
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<l;j++)	for(long i=0;i<n;i++)
-				aa.a[i+n*j] = a->v(i,k,j)*(1-d) + d*a->v(i,k+1,j);
-		a = &aa;
-	}
-	else
-	{	xx.Create(n,m);	yy.Create(n,m);	zz.Create(n,m);	}
-	yy.Fill(sv, sv);
-	xx.Fill(gr->Min.x, gr->Max.x,'x');
-	zz.Fill(gr->Min.z, gr->Max.z,'y');
+	a = fill_slice_y(gr,sv,a,xx,yy,zz,aa);
 #pragma omp parallel for
 	for(long i=0;i<v->GetNx();i++)
 	{
@@ -248,40 +208,21 @@ void MGL_EXPORT mgl_cont_y_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_cont_z_val(HMGL gr, HCDT v, HCDT a, const char *sch, double sv, const char *opt)
 {
-	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
-	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
+	long n=a->GetNx(),m=a->GetNy();
 	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"ContZ");	return;	}
-	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"ContZ");	return;	}
 	gr->SaveState(opt);
+	if(mgl_isnan(sv))	sv = gr->GetOrgZ('z');
+	if(sv<gr->Min.z || sv>gr->Max.z)	{	gr->SetWarn(mglWarnSlc,"ContZ");	gr->LoadState();	return;	}
 	static int cgid=1;	gr->StartGroup("ContZ",cgid++);
-	mglData xx,yy,zz,aa;
+	mglDataV xx,yy,zz;	mglData aa;
 
-	bool text=(mglchr(sch,'t'));
+	int text=0;
+	if(mglchr(sch,'t'))	text=1;
+	if(mglchr(sch,'T'))	text=2;
 	long ss=gr->AddTexture(sch);
 	gr->SetPenPal(sch);
 
-	xx.Create(n,m);	yy.Create(n,m);	zz.Create(n,m);
-	if(l>1)
-	{
-		aa.Create(n,m);
-		mreal d = (l-1)*(sv - gr->Min.z)/(gr->Max.z - gr->Min.z);
-		long k = long(d);	d = d - k;
-		if(k>l-2)	{	k=l-2;	d=1;	}
-		if(k<0)		{	k=0;	d=0;	}
-		const mglData *ma=dynamic_cast<const mglData *>(a);
-		if(ma)
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
-				aa.a[i+n*j] = ma->a[i+n*(j+m*k)]*(1-d) + d*ma->a[i+n*m+n*(j+m*k)];
-		else
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
-				aa.a[i+n*j] = a->v(i,j,k)*(1-d) + d*a->v(i,j,k+1);
-		a = &aa;
-	}
-	zz.Fill(sv, sv);
-	yy.Fill(gr->Min.y, gr->Max.y,'y');
-	xx.Fill(gr->Min.x, gr->Max.x,'x');
+	a = fill_slice_z(gr,sv,a,xx,yy,zz,aa);
 #pragma omp parallel for
 	for(long i=0;i<v->GetNx();i++)
 	{
@@ -355,38 +296,16 @@ void MGL_EXPORT mgl_cont_z_val_(uintptr_t *gr, uintptr_t *v, uintptr_t *a, const
 void MGL_EXPORT mgl_contf_gen(HMGL gr, mreal v1, mreal v2, HCDT a, HCDT x, HCDT y, HCDT z, mreal c, long ak);
 void MGL_EXPORT mgl_contf_x_val(HMGL gr, HCDT v, HCDT a, const char *sch, double sv, const char *opt)
 {
-	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
-	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
+	long n=a->GetNx(),m=a->GetNy();
 	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"ContFX");	return;	}
-	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"ContFX");	return;	}
 	gr->SaveState(opt);
+	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
+	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"ContFX");	gr->LoadState();	return;	}
 	static int cgid=1;	gr->StartGroup("ContFX",cgid++);
-	mglData xx,yy,zz,aa;
+	mglDataV xx,yy,zz;	mglData aa;
 	long ss=gr->AddTexture(sch);
 
-	if(l>1)
-	{
-		aa.Create(m,l);	xx.Create(m,l);	yy.Create(m,l);	zz.Create(m,l);
-		mreal d = (n-1)*(sv - gr->Min.x)/(gr->Max.x - gr->Min.x);
-		long k = long(d);	d = d - k;
-		if(k>n-2)	{	k=n-2;	d=1;	}
-		if(k<0)		{	k=0;	d=0;	}
-		const mglData *ma=dynamic_cast<const mglData *>(a);
-		if(ma)
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<l;j++)	for(long i=0;i<m;i++)
-				aa.a[i+m*j] = ma->a[k+n*(i+m*j)]*(1-d) + d*ma->a[k+1+n*(i+m*j)];
-		else
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<l;j++)	for(long i=0;i<m;i++)
-				aa.a[i+m*j] = a->v(k,i,j)*(1-d) + d*a->v(k+1,i,j);
-		a = &aa;
-	}
-	else
-	{	xx.Create(n,m);	yy.Create(n,m);	zz.Create(n,m);	}
-	xx.Fill(sv, sv);
-	yy.Fill(gr->Min.y, gr->Max.y,'x');
-	zz.Fill(gr->Min.z, gr->Max.z,'y');
+	a = fill_slice_x(gr,sv,a,xx,yy,zz,aa);
 #pragma omp parallel for
 	for(long i=0;i<v->GetNx()-1;i++)
 	{
@@ -398,38 +317,16 @@ void MGL_EXPORT mgl_contf_x_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contf_y_val(HMGL gr, HCDT v, HCDT a, const char *sch, double sv, const char *opt)
 {
-	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
-	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
+	long n=a->GetNx(),m=a->GetNy();
 	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"ContFY");	return;	}
-	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"ContFY");	return;	}
 	gr->SaveState(opt);
+	if(mgl_isnan(sv))	sv = gr->GetOrgY('y');
+	if(sv<gr->Min.y || sv>gr->Max.y)	{	gr->SetWarn(mglWarnSlc,"ContFY");	gr->LoadState();	return;	}
 	static int cgid=1;	gr->StartGroup("ContFY",cgid++);
-	mglData xx,yy,zz,aa;
+	mglDataV xx,yy,zz;	mglData aa;
 	long ss=gr->AddTexture(sch);
 
-	if(l>1)
-	{
-		aa.Create(n,l);	xx.Create(n,l);	yy.Create(n,l);	zz.Create(n,l);
-		mreal d = (m-1)*(sv - gr->Min.y)/(gr->Max.y - gr->Min.y);
-		long k = long(d);	d = d - k;
-		if(k>m-2)	{	k=m-2;	d=1;	}
-		if(k<0)		{	k=0;	d=0;	}
-		const mglData *ma=dynamic_cast<const mglData *>(a);
-		if(ma)
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<l;j++)	for(long i=0;i<n;i++)
-				aa.a[i+n*j] = ma->a[i+n*(k+m*j)]*(1-d) + d*ma->a[i+n+n*(k+m*j)];
-		else
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<l;j++)	for(long i=0;i<n;i++)
-				aa.a[i+n*j] = a->v(i,k,j)*(1-d) + d*a->v(i,k+1,j);
-		a = &aa;
-	}
-	else
-	{	xx.Create(n,m);	yy.Create(n,m);	zz.Create(n,m);	}
-	yy.Fill(sv, sv);
-	xx.Fill(gr->Min.x, gr->Max.x,'x');
-	zz.Fill(gr->Min.z, gr->Max.z,'y');
+	a = fill_slice_y(gr,sv,a,xx,yy,zz,aa);
 #pragma omp parallel for
 	for(long i=0;i<v->GetNx()-1;i++)
 	{
@@ -441,37 +338,16 @@ void MGL_EXPORT mgl_contf_y_val(HMGL gr, HCDT v, HCDT a, const char *sch, double
 //-----------------------------------------------------------------------------
 void MGL_EXPORT mgl_contf_z_val(HMGL gr, HCDT v, HCDT a, const char *sch, double sv, const char *opt)
 {
-	long n=a->GetNx(),m=a->GetNy(),l=a->GetNz();
-	if(mgl_isnan(sv))	sv = gr->GetOrgX('x');
+	long n=a->GetNx(),m=a->GetNy();
 	if(n<2 || m<2)	{	gr->SetWarn(mglWarnLow,"ContFZ");	return;	}
-	if(sv<gr->Min.x || sv>gr->Max.x)	{	gr->SetWarn(mglWarnSlc,"ContFZ");	return;	}
 	gr->SaveState(opt);
+	if(mgl_isnan(sv))	sv = gr->GetOrgZ('z');
+	if(sv<gr->Min.z || sv>gr->Max.z)	{	gr->SetWarn(mglWarnSlc,"ContFZ");	gr->LoadState();	return;	}
 	static int cgid=1;	gr->StartGroup("ContFZ",cgid++);
-	mglData xx,yy,zz,aa;
+	mglDataV xx,yy,zz;	mglData aa;
 	long ss=gr->AddTexture(sch);
 
-	xx.Create(n,m);	yy.Create(n,m);	zz.Create(n,m);
-	if(l>1)
-	{
-		aa.Create(n,m);
-		mreal d = (l-1)*(sv - gr->Min.z)/(gr->Max.z - gr->Min.z);
-		long k = long(d);	d = d - k;
-		if(k>l-2)	{	k=l-2;	d=1;	}
-		if(k<0)		{	k=0;	d=0;	}
-		const mglData *ma=dynamic_cast<const mglData *>(a);
-		if(ma)
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
-				aa.a[i+n*j] = ma->a[i+n*(j+m*k)]*(1-d) + d*ma->a[i+n*m+n*(j+m*k)];
-		else
-#pragma omp parallel for collapse(2)
-			for(long j=0;j<m;j++)	for(long i=0;i<n;i++)
-				aa.a[i+n*j] = a->v(i,j,k)*(1-d) + d*a->v(i,j,k+1);
-		a = &aa;
-	}
-	zz.Fill(sv, sv);
-	yy.Fill(gr->Min.y, gr->Max.y,'y');
-	xx.Fill(gr->Min.x, gr->Max.x,'x');
+	a = fill_slice_z(gr,sv,a,xx,yy,zz,aa);
 #pragma omp parallel for
 	for(long i=0;i<v->GetNx()-1;i++)
 	{
