@@ -1,6 +1,6 @@
 /***************************************************************************
  * parse.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -100,7 +100,7 @@ const mglCommand *mglParser::FindCommand(const wchar_t *com) const
 }
 //-----------------------------------------------------------------------------
 // return values : 0 -- OK, 1 -- wrong arguments, 2 -- wrong command, 3 -- unclosed string
-int mglParser::Exec(mglGraph *gr, const wchar_t *com, long n, mglArg *a, const std::wstring &var, const wchar_t *opt)
+int mglParser::Exec(mglGraph *gr, const wchar_t *com, long n, mglArg *a, const std::wstring &/*var*/, const wchar_t *opt)
 {
 	int i;
 	const char *id="dsn";
@@ -135,7 +135,7 @@ int mglParser::Exec(mglGraph *gr, const wchar_t *com, long n, mglArg *a, const s
 //-----------------------------------------------------------------------------
 mglParser::mglParser(bool setsize)
 {
-	InUse = 1;	curGr = 0;
+	InUse = 1;	curGr = 0;	Variant = 0;
 	Skip=Stop=for_br=false;
 	memset(for_stack,0,40*sizeof(int));
 	memset(if_stack,0,40*sizeof(int));
@@ -272,16 +272,17 @@ mglNum *mglParser::AddNum(const wchar_t *name)
 //-----------------------------------------------------------------------------
 int MGL_LOCAL_PURE mglFindArg(const std::wstring &str)
 {
-	register long l=0,k=0,i;
-	for(i=0;i<long(str.length());i++)
+	register long l=0,k=0;
+	const size_t s=str.length();
+	for(size_t i=0;i<s;i++)
 	{
 		if(str[i]=='\'') l++;
 		if(str[i]=='{') k++;
 		if(str[i]=='}') k--;
 		if(l%2==0 && k==0)
 		{
-			if(str[i]=='#' || str[i]==';')	return -i;
-			if(str[i]<=' ')	return i;
+			if(str[i]=='#' || str[i]==';')	return -long(i);
+			if(str[i]<=' ')	return long(i);
 		}
 	}
 	return 0;
@@ -298,11 +299,27 @@ void mglParser::FillArg(mglGraph *gr, int k, std::wstring *arg, mglArg *a)
 	{
 		mglDataA *v;	mglNum *f;
 		a[n-1].type = -1;
-		if(arg[n][0]=='|')	a[n-1].type = -1;
-		else if(arg[n][0]=='\'')	// this is string (simplest case)
+		std::wstring str = arg[n];
+		size_t i1=0, i2=str.length(), j=0;
+		bool s=true;
+		for(size_t i=0;i<i2;i++)
+		{
+			if(str[i]=='\'')	s = !s;
+			if(s && str[i]=='?')
+			{
+				if(j<Variant)	i1=i+1;
+				else	i2=i;
+				j++;
+			}
+		}
+		str = str.substr(i1,i2-i1);
+
+		if(str.empty())	a[n-1].type = -2;
+		else if(str[0]=='|')	a[n-1].type = -1;
+		else if(str[0]=='\'')	// this is string (simplest case)
 		{
 			a[n-1].type = 1;
-			std::wstring &w=arg[n],f;
+			std::wstring &w=str,f;
 			wchar_t buf[32];
 			long i,i1,ll=w.length();
 			for(i=1;i<ll;i++)
@@ -335,51 +352,51 @@ void mglParser::FillArg(mglGraph *gr, int k, std::wstring *arg, mglArg *a)
 				else	a[n-1].w += w[i];
 			}
 		}
-		else if(arg[n][0]=='{')
+		else if(str[0]=='{')
 		{	// this is temp data
 			mglData *u=new mglData;
-			std::wstring s = arg[n].substr(1,arg[n].length()-2);
+			std::wstring s = str.substr(1,str.length()-2);
 			a[n-1].w = u->s = L"/*"+s+L"*/";
 			a[n-1].type = 0;
 			ParseDat(gr, s, *u);	a[n-1].d = u;
 			u->temp=true;	DataList.push_back(u);
 		}
-		else if((v = FindVar(arg[n].c_str()))!=0)	// try to find normal variables (for data creation)
+		else if((v = FindVar(str.c_str()))!=0)	// try to find normal variables (for data creation)
 		{	a[n-1].type=0;	a[n-1].d=v;	a[n-1].w=v->s;	}
-		else if((f = FindNum(arg[n].c_str()))!=0)	// try to find normal number (for data creation)
+		else if((f = FindNum(str.c_str()))!=0)	// try to find normal number (for data creation)
 		{	a[n-1].type=2;	a[n-1].d=0;	a[n-1].v=f->d;	a[n-1].c=f->c;	a[n-1].w = f->s;	}
-		else if(arg[n][0]=='!')	// complex array is asked
+		else if(str[0]=='!')	// complex array is asked
 		{	// parse all numbers and formulas by unified way
-			HADT d = mglFormulaCalcC(arg[n].substr(1), this, DataList);
+			HADT d = mglFormulaCalcC(str.substr(1), this, DataList);
 			if(d->GetNN()==1)
 			{
-				if(CheckForName(arg[n].substr(1)))
+				if(CheckForName(str.substr(1)))
 				{	a[n-1].type = 2;	a[n-1].v = d->v(0);	a[n-1].c = d->a[0];	}
 				else
-				{	a[n-1].type = 0;	a[n-1].d = AddVar(arg[n].c_str());	}
+				{	a[n-1].type = 0;	a[n-1].d = AddVar(str.c_str());	}
 				delete d;
 			}
 			else
 			{
-				a[n-1].w = L"/*"+arg[n]+L"*/";
+				a[n-1].w = L"/*"+str+L"*/";
 				d->temp=true;	DataList.push_back(d);
 				a[n-1].type = 0;	a[n-1].d = d;
 			}
 		}
 		else
 		{	// parse all numbers and formulas by unified way
-			HMDT d = mglFormulaCalc(arg[n], this, DataList);
+			HMDT d = mglFormulaCalc(str, this, DataList);
 			if(d->GetNN()==1)
 			{
-				if(CheckForName(arg[n]))
+				if(CheckForName(str))
 				{	a[n-1].type = 2;	a[n-1].c = a[n-1].v = d->v(0);	}
 				else
-				{	a[n-1].type = 0;	a[n-1].d = AddVar(arg[n].c_str());	}
+				{	a[n-1].type = 0;	a[n-1].d = AddVar(str.c_str());	}
 				delete d;
 			}
 			else
 			{
-				a[n-1].w = L"/*"+arg[n]+L"*/";
+				a[n-1].w = L"/*"+str+L"*/";
 				d->temp=true;	DataList.push_back(d);
 				a[n-1].type = 0;	a[n-1].d = d;
 			}
@@ -573,7 +590,6 @@ int mglParser::Parse(mglGraph *gr, std::wstring str, long pos)
 {
 	if(Stop || gr->NeedStop())	return 0;
 	curGr = gr->Self();
-	std::wstring arg[1024];
 	str=mgl_trim_ws(str);
 	long n,k=0,m=0,mm=0,res;
 	// try parse ':' -- several commands in line
@@ -598,7 +614,8 @@ int mglParser::Parse(mglGraph *gr, std::wstring str, long pos)
 	PutArg(str,false);	str=mgl_trim_ws(str);
 
 	std::wstring opt;
-	for(k=0;k<1024;k++)	// parse string to substrings (by spaces)
+	std::vector<std::wstring> arg;
+	while(!str.empty())	// parse string to substrings (by spaces)
 	{
 		n = mglFindArg(str);
 		if(n<1)	// this is option
@@ -607,17 +624,18 @@ int mglParser::Parse(mglGraph *gr, std::wstring str, long pos)
 			if(n<0)	str = str.substr(0,-n);
 			break;
 		}
-		arg[k] = str.substr(0,n);
+		arg.push_back(str.substr(0,n));
 		str = mgl_trim_ws(str.substr(n+1));
 	}
 	// try to find last argument
-	if(str[0]!=0 && str[0]!='#' && str[0]!=';')	{	arg[k] = str;	k++;	}
-	if(k<1) n =0;
+	if(str[0]!=0 && str[0]!='#' && str[0]!=';')	arg.push_back(str);
+	k = arg.size();
+	if(k<1) n = 0;
 	else
 	{
 		// fill arguments by its values
 		mglArg *a = new mglArg[k];
-		FillArg(gr, k, arg, a);
+		FillArg(gr, k, &(arg[0]), a);
 		// execute first special (program-flow-control) commands
 		if(!skip() && !arg[0].compare(L"stop"))
 		{	Stop = true;	delete []a;	return 0;	}
@@ -656,10 +674,16 @@ int mglParser::Parse(mglGraph *gr, std::wstring str, long pos)
 				std::wstring a1 = arg[1], a2=arg[2];	res = 0;
 				if(a1[0]=='\'')	a1 = a1.substr(1,a1.length()-2);
 				if(a2[0]=='\'')	a2 = a2.substr(1,a2.length()-2);
-				mgl_rk_step_w(this, a1.c_str(), a2.c_str(), (k>=3 && a[2].type==2)?a[2].v:1);
+				mgl_rk_step_w(this, a1.c_str(), a2.c_str(), (k>3 && a[2].type==2)?a[2].v:1);
 			}
 			delete []a;	return res;
 		}
+		if(!arg[0].compare(L"variant"))
+		{
+			int res=1;
+			if(k==2 && a[0].type==2)	{	SetVariant(a[0].v);	res=0;	}
+			delete []a;	return res;
+		}
 		if(!arg[0].compare(L"call"))
 		{
 			n = 1;
@@ -702,6 +726,7 @@ int mglParser::Parse(mglGraph *gr, std::wstring str, long pos)
 		}
 		if(!arg[0].compare(L"for"))
 		{
+			if(k<2)	{	delete []a;	return 1;	}
 			n = 1;
 			char ch = arg[1][0];
 			int r = ch-'0';
@@ -737,10 +762,10 @@ int mglParser::Parse(mglGraph *gr, std::wstring str, long pos)
 			delete []a;	return n;
 		}
 		// alocate new arrays and execute the command itself
-		n = PreExec(gr, k, arg, a);
+		n = PreExec(gr, k, &(arg[0]), a);
 		if(n>0)	n--;
 		else if(!arg[0].compare(L"setsize") && !AllowSetSize)	n = 2;
-		else	n = Exec(gr, arg[0].c_str(),k-1,a, arg[1].c_str(), opt.c_str());
+		else	n = Exec(gr, arg[0].c_str(),k-1,a, k>1?arg[1]:L"", opt.c_str());
 		delete []a;
 	}
 	// delete temporary data arrays
@@ -887,7 +912,6 @@ int mglParser::FlowExec(mglGraph *, const std::wstring &com, long m, mglArg *a)
 	return n+1;
 }
 //-----------------------------------------------------------------------------
-#include <string>
 void mglParser::Execute(mglGraph *gr, FILE *fp, bool print)
 {
 	if(gr==0 || fp==0)	return;
@@ -1303,3 +1327,6 @@ void MGL_EXPORT mgl_rk_step_(uintptr_t *p, const char *eqs, const char *vars, do
 	char *s=new char[m+1];	memcpy(s,vars,m);	s[m]=0;
 	mgl_rk_step(_PR_,e,s,*dt);	delete []e;	delete []s;	}
 //---------------------------------------------------------------------------
+void MGL_EXPORT mgl_parser_variant(HMPR p, int var)	{	p->SetVariant(var);	}
+void MGL_EXPORT mgl_parser_variant_(uintptr_t *p, int *var)	{	mgl_parser_variant(_PR_,*var);	}
+//---------------------------------------------------------------------------
