@@ -1146,34 +1146,34 @@ uintptr_t MGL_EXPORT mgl_jacobian_3d_(uintptr_t* x, uintptr_t* y, uintptr_t* z)
 //	Progonka
 //
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_progonka_sr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, long n, long i0, long di, bool difr)
+void MGL_NO_EXPORT mgl_progonka_sr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, long n, long id, long i0, long di, bool difr)
 {
 	mreal *aa=dat, *bb=dat+n, *uu=dat+2*n;
-	mreal b0=B->vthr(i0), c0=C->vthr(i0), d0=D->vthr(i0);
-	if(difr)	d0 = (2.-b0)*d0-c0*D->vthr(i0+di);
+	mreal b0=B->vthr(i0), c0=C->vthr(i0), d0=D->vthr(id);
+	if(difr)	d0 = (2.-b0)*d0-c0*D->vthr(id+di);
 	aa[0] = -c0/b0;	bb[0] = d0/b0;
 	for(long i=1;i<n;i++)
 	{
-		register long ii=i0+di*i;
+		register long ii=i0+di*i, dd=id+di*i, tt = id+di*((i+1)%n);
 		mreal a=A->vthr(ii), b=B->vthr(ii), c=C->vthr(ii);
-		mreal d=difr?-a*D->vthr(ii-di)+(2.-b)*D->vthr(ii)-c*D->vthr(ii+di):D->vthr(ii);
+		mreal d=difr?-a*D->vthr(dd-di)+(2.-b)*D->vthr(dd)-c*D->vthr(tt):D->vthr(dd);
 		aa[i] = -c/(b+a*aa[i-1]);
 		bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
 	}
 	uu[n-1] = bb[n-1];
 	for(long i=n-2;i>=0;i--)	uu[i] = bb[i]+aa[i]*uu[i+1];
 }
-void MGL_NO_EXPORT mgl_progonka_pr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, long n, long i0, long di, bool difr)
+void MGL_NO_EXPORT mgl_progonka_pr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, long n, long id, long i0, long di, bool difr)
 {
 	mreal *aa=dat, *bb=dat+n, *gg=dat+2*n, *uu=dat+3*n;
-	mreal a0=A->vthr(i0), b0=B->vthr(i0), c0=C->vthr(i0), d0=D->vthr(i0);
-	if(difr)	d0 = -a0*D->vthr(i0+di*(n-1))+(2.-b0)*d0-c0*D->vthr(i0+di);
+	mreal a0=A->vthr(i0), b0=B->vthr(i0), c0=C->vthr(i0), d0=D->vthr(id);
+	if(difr)	d0 = -a0*D->vthr(id+di*(n-1))+(2.-b0)*d0-c0*D->vthr(id+di);
 	aa[0] =-c0/b0;	bb[0] = d0/b0;	gg[0] =-a0/b0;
 	for(long i=1;i<n;i++)
 	{
-		register long ii=i0+di*i, il=i0+di*((i+1)%n);
+		register long ii=i0+di*i, il=id+di*((i+1)%n), dd=id+di*i;
 		mreal a=A->vthr(ii), b=B->vthr(ii), c=C->vthr(ii);
-		mreal d=difr?-a*D->vthr(ii-di)+(2.-b)*D->vthr(ii)-c*D->vthr(il):D->vthr(ii);
+		mreal d=difr?-a*D->vthr(dd-di)+(2.-b)*D->vthr(dd)-c*D->vthr(il):D->vthr(dd);
 		aa[i] = -c/(b+a*aa[i-1]);
 		bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
 		gg[i] = -a*gg[i-1]/(b+a*aa[i-1]);
@@ -1188,26 +1188,26 @@ void MGL_NO_EXPORT mgl_progonka_pr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, l
 	mreal u0 = bb[0]/(1.-aa[0]);
 	for(long i=0;i<n;i++)	uu[i]=bb[i]+aa[i]*u0;
 }
-void MGL_NO_EXPORT mgl_progonka_hr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, long n, long i0, bool difr)
+void MGL_NO_EXPORT mgl_progonka_hr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, long n, long id, long i0, bool difr)
 {
 	mreal *aa=dat, *bb=dat+n, *uu=dat+n*n;
-	mreal b0=B->vthr(i0), c0=C->vthr(i0), d0=D->vthr(i0);
+	mreal b0=B->vthr(i0), c0=C->vthr(i0), d0=D->vthr(id);
 	uu[0] = d0/b0*(difr?(2.-b0):1.);
-	b0=B->vthr(i0+n*n-1);	d0=D->vthr(i0+n*n-1);
+	b0=B->vthr(i0+n*n-1);	d0=D->vthr(id+n*n-1);
 	uu[n*n-1] = d0/b0*(difr?(2.-b0):1.);
-	long di = n-1, i1 = i0+n*(n-1);
+	long di = n-1, i1 = i0+n*(n-1), d1 = id+n*(n-1);
 	// suppose the square grid!
 	for(long j=1;j<n;j++)
 	{
 		// first bottom-left triangle
-		b0=B->vthr(i0+j);	c0=C->vthr(i0+j);	d0=D->vthr(i0+j);
-		if(difr)	d0 = (2.-b0)*d0-c0*D->vthr(i0+j+di);
+		b0=B->vthr(i0+j);	c0=C->vthr(i0+j);	d0=D->vthr(id+j);
+		if(difr)	d0 = (2.-b0)*d0-c0*D->vthr(id+j+di);
 		aa[0] = -c0/b0;	bb[0] = d0/b0;
 		for(long i=1;i<=j;i++)
 		{
-			register long ii=i0+j+di*i;
+			register long ii=i0+j+di*i, dd=id+j+di*i;
 			mreal a=A->vthr(ii),b=B->vthr(ii),c=C->vthr(ii);
-			mreal d=difr?-a*D->vthr(ii-di)+(2.-b)*D->vthr(ii)-c*D->vthr(ii+di):D->vthr(ii);
+			mreal d=difr?-a*D->vthr(dd-di)+(2.-b)*D->vthr(dd)-c*D->vthr(dd+di):D->vthr(dd);
 			aa[i] = -c/(b+a*aa[i-1]);
 			bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
 		}
@@ -1216,14 +1216,14 @@ void MGL_NO_EXPORT mgl_progonka_hr(HCDT A, HCDT B, HCDT C, HCDT D, mreal *dat, l
 			uu[j+di*i] = bb[i]+aa[i]*uu[j+di*i+di];
 		// next top-right triangle
 		long j1=n-1-j;
-		b0=B->vthr(i1+j1);	c0=C->vthr(i1+j1);	d0=D->vthr(i1+j1);
-		if(difr)	d0 = (2.-b0)*d0-c0*D->vthr(i1+j1-di);
+		b0=B->vthr(i1+j1);	c0=C->vthr(i1+j1);	d0=D->vthr(d1+j1);
+		if(difr)	d0 = (2.-b0)*d0-c0*D->vthr(d1+j1-di);
 		aa[0] = -c0/b0;	bb[0] = d0/b0;
 		for(long i=1;i<=j;i++)
 		{
-			register long ii=i1+j1-di*i;
+			register long ii=i1+j1-di*i, dd=d1+j1-di*i;
 			mreal a=A->vthr(ii),b=B->vthr(ii),c=C->vthr(ii);
-			mreal d=difr?-a*D->vthr(ii+di)+(2.-b)*D->vthr(ii)-c*D->vthr(ii-di):D->vthr(ii);
+			mreal d=difr?-a*D->vthr(dd+di)+(2.-b)*D->vthr(dd)-c*D->vthr(dd-di):D->vthr(dd);
 			aa[i] = -c/(b+a*aa[i-1]);
 			bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
 		}
@@ -1248,10 +1248,10 @@ HMDT MGL_EXPORT mgl_data_tridmat(HCDT A, HCDT B, HCDT C, HCDT D, const char *how
 #pragma omp for collapse(2)
 		for(long k=0;k<nz;k++)	for(long j=0;j<ny;j++)
 		{
-			long i0=0;
+			long i0=0, i1=nx*(j+ny*k);
 			if(na==nn)	i0=nx*(j+ny*k);	else if(na==np)	i0=nx*j;
-			if(per)	mgl_progonka_pr(A,B,C,D,T.a,nx,i0,1,difr);
-			else	mgl_progonka_sr(A,B,C,D,T.a,nx,i0,1,difr);
+			if(per)	mgl_progonka_pr(A,B,C,D,T.a,nx,i1,i0,1,difr);
+			else	mgl_progonka_sr(A,B,C,D,T.a,nx,i1,i0,1,difr);
 			i0 = nx*(j+ny*k);
 			for(long i=0;i<nx;i++)	r->a[i+i0] = uu[i];
 		}
@@ -1263,10 +1263,10 @@ HMDT MGL_EXPORT mgl_data_tridmat(HCDT A, HCDT B, HCDT C, HCDT D, const char *how
 #pragma omp for collapse(2)
 		for(long k=0;k<nz;k++)	for(long i=0;i<nx;i++)
 		{
-			long i0=0;
+			long i0=0, i1 = i+np*k;
 			if(na==nn)	i0=i+np*k;	else if(na==np)	i0=i;
-			if(per)	mgl_progonka_pr(A,B,C,D,T.a,ny,i0,nx,difr);
-			else	mgl_progonka_sr(A,B,C,D,T.a,ny,i0,nx,difr);
+			if(per)	mgl_progonka_pr(A,B,C,D,T.a,ny,i1,i0,nx,difr);
+			else	mgl_progonka_sr(A,B,C,D,T.a,ny,i1,i0,nx,difr);
 			i0 = i+np*k;
 			for(long j=0;j<ny;j++)	r->a[j*nx+i0] = uu[j];
 		}
@@ -1278,9 +1278,9 @@ HMDT MGL_EXPORT mgl_data_tridmat(HCDT A, HCDT B, HCDT C, HCDT D, const char *how
 #pragma omp for collapse(2)
 		for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
 		{
-			long i0 = na==nn?i+nx*j:0;
-			if(per)	mgl_progonka_pr(A,B,C,D,T.a,nz,i0,np,difr);
-			else	mgl_progonka_sr(A,B,C,D,T.a,nz,i0,np,difr);
+			long i0 = na==nn?i+nx*j:0, i1 = i+nx*j;
+			if(per)	mgl_progonka_pr(A,B,C,D,T.a,nz,i1,i0,np,difr);
+			else	mgl_progonka_sr(A,B,C,D,T.a,nz,i1,i0,np,difr);
 			i0 = i+nx*j;
 			for(long k=0;k<nz;k++)	r->a[k*np+i0] = uu[k];
 		}
@@ -1292,7 +1292,7 @@ HMDT MGL_EXPORT mgl_data_tridmat(HCDT A, HCDT B, HCDT C, HCDT D, const char *how
 #pragma omp for
 		for(long k=0;k<nz;k++)
 		{
-			mgl_progonka_hr(A,B,C,D,T.a,nx,na==nn ? k*np:0,difr);
+			mgl_progonka_hr(A,B,C,D,T.a,nx,k*np,na==nn ? k*np:0,difr);
 			memcpy(r->a+k*np, T.a+np, np*sizeof(mreal));
 		}
 	}
@@ -1306,34 +1306,34 @@ uintptr_t MGL_EXPORT mgl_data_tridmat_(uintptr_t *A, uintptr_t *B, uintptr_t *C,
 	delete []s;	return r;
 }
 //-----------------------------------------------------------------------------
-void MGL_NO_EXPORT mgl_progonka_sc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, long n, long i0, long di, bool difr)
+void MGL_NO_EXPORT mgl_progonka_sc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, long n, long id, long i0, long di, bool difr)
 {
 	dual *aa=dat, *bb=dat+n, *uu=dat+2*n;
-	dual b0=B->vcthr(i0), c0=C->vcthr(i0), d0=D->vcthr(i0);
-	if(difr)	d0 = (2.-b0)*d0-c0*D->vcthr(i0+di);
+	dual b0=B->vcthr(i0), c0=C->vcthr(i0), d0=D->vcthr(id);
+	if(difr)	d0 = (2.-b0)*d0-c0*D->vcthr(id+di);
 	aa[0] = -c0/b0;	bb[0] = d0/b0;
 	for(long i=1;i<n;i++)
 	{
-		register long ii=i0+di*i;
+		register long ii=i0+di*i, dd=id+di*i, tt = id+di*((i+1)%n);
 		dual a=A->vcthr(ii), b=B->vcthr(ii), c=C->vcthr(ii);
-		dual d=difr?-a*D->vcthr(ii-di)+(2.-b)*D->vcthr(ii)-c*D->vcthr(ii+di):D->vcthr(ii);
+		dual d=difr?-a*D->vcthr(dd-di)+(2.-b)*D->vcthr(dd)-c*D->vcthr(tt):D->vcthr(dd);
 		aa[i] = -c/(b+a*aa[i-1]);
 		bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
 	}
 	uu[n-1] = bb[n-1];
 	for(long i=n-2;i>=0;i--)	uu[i] = bb[i]+aa[i]*uu[i+1];
 }
-void MGL_NO_EXPORT mgl_progonka_pc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, long n, long i0, long di, bool difr)
+void MGL_NO_EXPORT mgl_progonka_pc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, long n, long id, long i0, long di, bool difr)
 {
 	dual *aa=dat, *bb=dat+n, *gg=dat+2*n, *uu=dat+3*n;
-	dual a0=A->vcthr(i0), b0=B->vcthr(i0), c0=C->vcthr(i0), d0=D->vcthr(i0);
-	if(difr)	d0 = -a0*D->vcthr(i0+di*(n-1))+(2.-b0)*d0-c0*D->vcthr(i0+di);
+	dual a0=A->vcthr(i0), b0=B->vcthr(i0), c0=C->vcthr(i0), d0=D->vcthr(id);
+	if(difr)	d0 = -a0*D->vcthr(id+di*(n-1))+(2.-b0)*d0-c0*D->vcthr(id+di);
 	aa[0] =-c0/b0;	bb[0] = d0/b0;	gg[0] =-a0/b0;
 	for(long i=1;i<n;i++)
 	{
-		register long ii=i0+di*i, il=i0+di*((i+1)%n);
+		register long ii=i0+di*i, il=id+di*((i+1)%n), dd=id+di*i;
 		dual a=A->vcthr(ii), b=B->vcthr(ii), c=C->vcthr(ii);
-		dual d=difr?-a*D->vcthr(ii-di)+(2.-b)*D->vcthr(ii)-c*D->vcthr(il):D->vcthr(ii);
+		dual d=difr?-a*D->vcthr(dd-di)+(2.-b)*D->vcthr(dd)-c*D->vcthr(il):D->vcthr(dd);
 		aa[i] = -c/(b+a*aa[i-1]);
 		bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
 		gg[i] = -a*gg[i-1]/(b+a*aa[i-1]);
@@ -1348,26 +1348,26 @@ void MGL_NO_EXPORT mgl_progonka_pc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, lo
 	dual u0 = bb[0]/(1.-aa[0]);
 	for(long i=0;i<n;i++)	uu[i]=bb[i]+aa[i]*u0;
 }
-void MGL_NO_EXPORT mgl_progonka_hc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, long n, long i0, bool difr)
+void MGL_NO_EXPORT mgl_progonka_hc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, long n, long id, long i0, bool difr)
 {
 	dual *aa=dat, *bb=dat+n, *uu=dat+n*n;
-	dual b0=B->vcthr(i0), c0=C->vcthr(i0), d0=D->vcthr(i0);
+	dual b0=B->vcthr(i0), c0=C->vcthr(i0), d0=D->vcthr(id);
 	uu[0] = d0/b0*(difr?(2.-b0):1.);
-	b0=B->vcthr(i0+n*n-1);	d0=D->vcthr(i0+n*n-1);
+	b0=B->vcthr(i0+n*n-1);	d0=D->vcthr(id+n*n-1);
 	uu[n*n-1] = d0/b0*(difr?(2.-b0):1.);
-	long di = n-1, i1 = i0+n*(n-1);
+	long di = n-1, i1 = i0+n*(n-1), d1 = id+n*(n-1);
 	// suppose the square grid!
 	for(long j=1;j<n;j++)
 	{
 		// first bottom-left triangle
-		b0=B->vcthr(i0+j);	c0=C->vcthr(i0+j);	d0=D->vcthr(i0+j);
-		if(difr)	d0 = (2.-b0)*d0-c0*D->vcthr(i0+j+di);
+		b0=B->vcthr(i0+j);	c0=C->vcthr(i0+j);	d0=D->vcthr(id+j);
+		if(difr)	d0 = (2.-b0)*d0-c0*D->vcthr(id+j+di);
 		aa[0] = -c0/b0;	bb[0] = d0/b0;
 		for(long i=1;i<=j;i++)
 		{
-			register long ii=i0+j+di*i;
+			register long ii=i0+j+di*i, dd=id+j+di*i;
 			dual a=A->vcthr(ii),b=B->vcthr(ii),c=C->vcthr(ii);
-			dual d=difr?-a*D->vcthr(ii-di)+(2.-b)*D->vcthr(ii)-c*D->vcthr(ii+di):D->vcthr(ii);
+			dual d=difr?-a*D->vcthr(dd-di)+(2.-b)*D->vcthr(dd)-c*D->vcthr(dd+di):D->vcthr(dd);
 			aa[i] = -c/(b+a*aa[i-1]);
 			bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
 		}
@@ -1376,14 +1376,14 @@ void MGL_NO_EXPORT mgl_progonka_hc(HCDT A, HCDT B, HCDT C, HCDT D, dual *dat, lo
 			uu[j+di*i] = bb[i]+aa[i]*uu[j+di*i+di];
 		// next top-right triangle
 		long j1=n-1-j;
-		b0=B->vcthr(i1+j1);	c0=C->vcthr(i1+j1);	d0=D->vcthr(i1+j1);
-		if(difr)	d0 = (2.-b0)*d0-c0*D->vcthr(i1+j1-di);
+		b0=B->vcthr(i1+j1);	c0=C->vcthr(i1+j1);	d0=D->vcthr(d1+j1);
+		if(difr)	d0 = (2.-b0)*d0-c0*D->vcthr(d1+j1-di);
 		aa[0] = -c0/b0;	bb[0] = d0/b0;
 		for(long i=1;i<=j;i++)
 		{
-			register long ii=i1+j1-di*i;
+			register long ii=i1+j1-di*i, dd=d1+j1-di*i;
 			dual a=A->vcthr(ii),b=B->vcthr(ii),c=C->vcthr(ii);
-			dual d=difr?-a*D->vcthr(ii+di)+(2.-b)*D->vcthr(ii)-c*D->vcthr(ii-di):D->vcthr(ii);
+			dual d=difr?-a*D->vcthr(dd+di)+(2.-b)*D->vcthr(dd)-c*D->vcthr(dd-di):D->vcthr(dd);
 			aa[i] = -c/(b+a*aa[i-1]);
 			bb[i] = (d-a*bb[i-1])/(b+a*aa[i-1]);
 		}
@@ -1408,12 +1408,11 @@ HADT MGL_EXPORT mgl_datac_tridmat(HCDT A, HCDT B, HCDT C, HCDT D, const char *ho
 #pragma omp for collapse(2)
 		for(long k=0;k<nz;k++)	for(long j=0;j<ny;j++)
 		{
-			long i0=0;
-			if(na==nn)	i0=nx*(j+ny*k);	else if(na==np)	i0=nx*j;
-			if(per)	mgl_progonka_pc(A,B,C,D,T.a,nx,i0,1,difr);
-			else	mgl_progonka_sc(A,B,C,D,T.a,nx,i0,1,difr);
-			i0 = nx*(j+ny*k);
-			for(long i=0;i<nx;i++)	r->a[i+i0] = uu[i];
+			long i0=0, i1=nx*(j+ny*k);
+			if(na==nn)	i0=i1;	else if(na==np)	i0=nx*j;
+			if(per)	mgl_progonka_pc(A,B,C,D,T.a,nx,i1,i0,1,difr);
+			else	mgl_progonka_sc(A,B,C,D,T.a,nx,i1,i0,1,difr);
+			for(long i=0;i<nx;i++)	r->a[i+i1] = uu[i];
 		}
 	}
 	else if(mglchr(how,'y') && (na==nn || na==np || na==ny))
@@ -1423,10 +1422,10 @@ HADT MGL_EXPORT mgl_datac_tridmat(HCDT A, HCDT B, HCDT C, HCDT D, const char *ho
 #pragma omp for collapse(2)
 		for(long k=0;k<nz;k++)	for(long i=0;i<nx;i++)
 		{
-			long i0=0;
-			if(na==nn)	i0=i+np*k;	else if(na==np)	i0=i;
-			if(per)	mgl_progonka_pc(A,B,C,D,T.a,ny,i0,nx,difr);
-			else	mgl_progonka_sc(A,B,C,D,T.a,ny,i0,nx,difr);
+			long i0=0, i1 = i+np*k;
+			if(na==nn)	i0=i1;	else if(na==np)	i0=i;
+			if(per)	mgl_progonka_pc(A,B,C,D,T.a,ny,i1,i0,nx,difr);
+			else	mgl_progonka_sc(A,B,C,D,T.a,ny,i1,i0,nx,difr);
 			i0 = i+np*k;
 			for(long j=0;j<ny;j++)	r->a[j*nx+i0] = uu[j];
 		}
@@ -1438,11 +1437,10 @@ HADT MGL_EXPORT mgl_datac_tridmat(HCDT A, HCDT B, HCDT C, HCDT D, const char *ho
 #pragma omp for collapse(2)
 		for(long j=0;j<ny;j++)	for(long i=0;i<nx;i++)
 		{
-			long i0 = na==nn?i+nx*j:0;
-			if(per)	mgl_progonka_pc(A,B,C,D,T.a,nz,i0,np,difr);
-			else	mgl_progonka_sc(A,B,C,D,T.a,nz,i0,np,difr);
-			i0 = i+nx*j;
-			for(long k=0;k<nz;k++)	r->a[k*np+i0] = uu[k];
+			long i0 = na==nn?i+nx*j:0, i1 = i+nx*j;
+			if(per)	mgl_progonka_pc(A,B,C,D,T.a,nz,i1,i0,np,difr);
+			else	mgl_progonka_sc(A,B,C,D,T.a,nz,i1,i0,np,difr);
+			for(long k=0;k<nz;k++)	r->a[k*np+i1] = uu[k];
 		}
 	}
 	else if(mglchr(how,'h') && ny==nx && (na==nn || na==np) && nx>1)
@@ -1452,7 +1450,7 @@ HADT MGL_EXPORT mgl_datac_tridmat(HCDT A, HCDT B, HCDT C, HCDT D, const char *ho
 #pragma omp for
 		for(long k=0;k<nz;k++)
 		{
-			mgl_progonka_hc(A,B,C,D,T.a,nx,na==nn ? k*np:0,difr);
+			mgl_progonka_hc(A,B,C,D,T.a,nx,k*np, na==nn ? k*np:0,difr);
 			memcpy(r->a+k*np, T.a+np, np*sizeof(dual));
 		}
 	}
