@@ -1,6 +1,6 @@
 /***************************************************************************
  * pixel.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -20,6 +20,9 @@
 #include <algorithm>
 #include "mgl2/canvas.h"
 #include "mgl2/thread.h"
+#if MGL_HAVE_OMP
+#include <omp.h>
+#endif
 
 inline mreal get_persp(float pf, float z, float Depth)
 //{	return (1-pf)/(1-pf*z/Depth);	}
@@ -46,7 +49,7 @@ void mglCanvas::SetSize(int w,int h,bool clf)
 #if MGL_HAVE_PTHREAD
 	pthread_mutex_lock(&mutexClf);
 #elif MGL_HAVE_OMP
-	omp_set_lock(&lockClf);
+	omp_set_lock((omp_lock_t*)lockClf);
 #endif
 	if(G)	{	delete []G;	delete []C;	delete []Z;	delete []G4;delete []GB;delete []OI;	G=0;	}
 	G = new unsigned char[s*3];
@@ -60,7 +63,7 @@ void mglCanvas::SetSize(int w,int h,bool clf)
 #if MGL_HAVE_PTHREAD
 	pthread_mutex_unlock(&mutexClf);
 #elif MGL_HAVE_OMP
-	omp_unset_lock(&lockClf);
+	omp_unset_lock((omp_lock_t*)lockClf);
 #endif
 
 	InPlot(0,1,0,1,false);
@@ -71,7 +74,7 @@ void mglCanvas::SetSize(int w,int h,bool clf)
 		pthread_mutex_lock(&mutexPnt);
 		pthread_mutex_lock(&mutexClf);
 #elif MGL_HAVE_OMP
-		omp_set_lock(&lockClf);
+		omp_set_lock((omp_lock_t*)lockClf);
 #endif
 		const long m = long(Prm.size());
 		double dd = dx>dy?dy:dx;
@@ -115,7 +118,7 @@ void mglCanvas::SetSize(int w,int h,bool clf)
 		pthread_mutex_unlock(&mutexClf);
 		pthread_mutex_unlock(&mutexPnt);
 #elif MGL_HAVE_OMP
-		omp_unset_lock(&lockClf);
+		omp_unset_lock((omp_lock_t*)lockClf);
 #endif
 		ClfZB();	Finish();
 	}
@@ -672,7 +675,7 @@ void mglCanvas::Finish()
 	pthread_mutex_lock(&mutexPnt);
 	pthread_mutex_lock(&mutexClf);
 #elif MGL_HAVE_OMP
-	omp_set_lock(&lockClf);
+	omp_set_lock((omp_lock_t*)lockClf);
 #endif
 	if(Quality==MGL_DRAW_DOTS)
 	{
@@ -706,7 +709,7 @@ void mglCanvas::Finish()
 	pthread_mutex_unlock(&mutexPnt);
 	pthread_mutex_unlock(&mutexPrm);
 #elif MGL_HAVE_OMP
-	omp_unset_lock(&lockClf);
+	omp_unset_lock((omp_lock_t*)lockClf);
 #endif
 }
 //-----------------------------------------------------------------------------
@@ -912,7 +915,6 @@ void mglCanvas::combine(unsigned char *c1, const unsigned char *c2) const
 	if(c2[3])
 	{
 		const register unsigned a1=c1[3], a2=c2[3];
-		if(a2==255 || a1==0)	{	memcpy(c1,c2,4);	return; }
 		if((Flag&3)==0)
 		{
 			register unsigned b1=255-a2;
@@ -931,10 +933,10 @@ void mglCanvas::combine(unsigned char *c1, const unsigned char *c2) const
 		else if((Flag&3)==2)
 		{
 			register unsigned b1,b2,b3;
-			b1 = (c1[0]*a1 + c2[0]*a2)/256;		c1[0] = b1<255 ? b1 : 255;
-			b2 = (c1[1]*a1 + c2[1]*a2)/256;		c1[1] = b2<255 ? b2 : 255;
-			b3 = (c1[2]*a1 + c2[2]*a2)/256;		c1[2] = b3<255 ? b3 : 255;
-			c1[3] = a1+a2>255? 255 : a1+a2;
+			b1 = (c1[0]*a1 + c2[0]*a2)/255;	c1[0] = b1<255 ? b1 : 255;
+			b2 = (c1[1]*a1 + c2[1]*a2)/255;	c1[1] = b2<255 ? b2 : 255;
+			b3 = (c1[2]*a1 + c2[2]*a2)/255;	c1[2] = b3<255 ? b3 : 255;
+			c1[3] = 255;
 		}
 	}
 }
@@ -1193,7 +1195,7 @@ void mglCanvas::line_draw(const mglPnt &p1, const mglPnt &p2, const mglDrawReg *
 			if(u<0)			v += u*u;
 			else if(u>dd)	v += (u-dd)*(u-dd);
 //			if(v>pw*pw)		continue;
-			if(!(pd & ( 1L<<long(fmod(pp+u/pw/1.5, 16)) ) ))	continue;
+			if(!(pd & ((uint64_t)1<<long(fmod(pp+u/pw/1.5, 16)) ) ))	continue;
 			mglPnt p(p1+d*(u/dd));	col2int(p,r,oi);
 			r[3] = v<(pw-1)*(pw-1)/4 ? 255 : mgl_sline(255,dpw*(sqrt(v)+(1-pw)/2));
 			pnt_plot(i,j,p.z+dz,r,oi);
@@ -1213,7 +1215,7 @@ void mglCanvas::line_draw(const mglPnt &p1, const mglPnt &p2, const mglDrawReg *
 			if(u<0)			v += u*u;
 			else if(u>dd)	v += (u-dd)*(u-dd);
 //			if(v>pw*pw)		continue;
-			if(!(pd & (1L<<long(fmod(pp+u/pw/1.5, 16)))))		continue;
+			if(!(pd & ((uint64_t)1<<long(fmod(pp+u/pw/1.5, 16)))))		continue;
 			mglPnt p(p1+d*(u/dd));	col2int(p,r,oi);
 			r[3] = v<(pw-1)*(pw-1)/4 ? 255 : mgl_sline(255,dpw*(sqrt(v)+(1-pw)/2));
 			pnt_plot(i,j,p.z+dz,r,oi);
@@ -1270,7 +1272,7 @@ void mglCanvas::line_pix(long i, long j, const mglPnt &p1, const mglPnt &p2, con
 	else if(u>dd)	v += (u-dd)*(u-dd);
 	register float pw=dr->PenWidth, dpw=3*pen_delta;
 	if(dr->ObjId==HighId)	{	pw *= 2;	dpw=2*pen_delta;	}
-	if(v>pw*pw || !(dr->PDef & ( 1L<<long(fmod(dr->pPos+u/pw/1.5, 16)) ) ))	return;
+	if(v>pw*pw || !(dr->PDef & ( (uint64_t)1<<long(fmod(dr->pPos+u/pw/1.5, 16)) ) ))	return;
 	mglPnt p(p1+d*(u/dd));
 	unsigned char r[4];
 	col2int(p,r,dr->ObjId);
