@@ -98,30 +98,40 @@ void DatPanel::refresh()
 		}
 		tab->setHorizontalHeaderLabels(head);
 	}
-	register long i,j,m=var->s.length();
-	register mreal f;
+	long m=var->s.length();
 	QString s,d;
 	if(rc)
 	{
 		QStringList sh,sv;
-		for(i=0;i<nx;i++)	sh<<QString::number(i);
+		for(long i=0;i<nx;i++)	sh<<QString::number(i);
 		tab->setHorizontalHeaderLabels(sh);
-		for(i=0;i<ny;i++)	sv<<QString::number(i);
+		for(long i=0;i<ny;i++)	sv<<QString::number(i);
 		tab->setVerticalHeaderLabels(sv);
-		for(i=0;i<nx;i++)	for(j=0;j<ny;j++)
+		for(long i=0;i<nx;i++)	for(long j=0;j<ny;j++)
 			tab->setItem(j,i,new QTableWidgetItem);
 	}
-	for(i=0;i<nx;i++)	for(j=0;j<ny;j++)
+	mglDataC *cc = dynamic_cast<mglDataC*>(var);
+	if(cc)	for(long i=0;i<nx;i++)	for(long j=0;j<ny;j++)
 	{
-		f = var->v(i,j,kz);
+		dual f = cc->a[i+nx*(j+ny*kz)];
+		if(mgl_isnan(f))	s = "nan";
+		else if(mgl_isbad(f))	s="inf";
+		else if(imag(f)>0)	s.sprintf("%.15g+%.15gi",real(f),imag(f));
+		else if(imag(f)<0)	s.sprintf("%.15g-%.15gi",real(f),-imag(f));
+		else	s.sprintf("%15g",real(f));
+		tab->item(j,i)->setText(s);
+	}
+	else	for(long i=0;i<nx;i++)	for(long j=0;j<ny;j++)
+	{
+		double f = var->v(i,j,kz);
 		if(mgl_isnan(f))	s = "nan";
 		else if(mgl_isbad(f))	s=f>0?"inf":"-inf";
-		else	s.sprintf("%g",f);
+		else	s.sprintf("%.15g",f);
 		tab->item(j,i)->setText(s);
 	}
 	infoDlg->allowRefresh=true;	infoDlg->refresh();
 	QChar *ss = new QChar[m+1];
-	for(i=0;i<m;i++)	ss[i] = var->s[i];
+	for(long i=0;i<m;i++)	ss[i] = var->s[i];
 	s = QString(ss, m);	delete []ss;
 	d.sprintf("%d * %d * %d", nx, ny, nz);
 	ready = true;
@@ -135,7 +145,7 @@ void DatPanel::setVar(mglDataA *v)
 	nx = ny = nz = kz = 0;
 	if(v)
 	{
-		QString s = QString::fromStdWString(v->s);
+		QString s = QString::fromWCharArray(v->s.c_str());
 		v->o = this;	v->func = deleteDat;
 		refresh();
 		setWindowTitle(s + tr(" - UDAV variable"));
@@ -157,23 +167,77 @@ void DatPanel::setSlice(int k)
 	}
 }
 //-----------------------------------------------------------------------------
+dual mgl_str2dual(const char *s)
+{
+	setlocale(LC_NUMERIC, "C");
+	double re=0,im=0;	size_t ll=strlen(s);
+	while(s[ll]<=' ')	ll--;
+	if(*s=='(')		sscanf(s,"(%lg,%lg)",&re,&im);
+	else if(*s=='i')	{	re=0;	im=atof(s+1);	}
+	else if(*s=='[')	sscanf(s,"[%lg,%lg]",&re,&im);
+	else if(*s=='{')	sscanf(s,"{%lg,%lg}",&re,&im);
+	else if(s[ll]=='i')
+	{
+		double a,b;
+		int s1=sscanf(s,"%lg+%lgi",&re,&im);
+		int s2=sscanf(s,"%lg-%lgi",&a,&b);
+		if(s1<2)
+		{
+		if(s2==2)	{	re=a;	im=-b;	}
+		else	{	im=atof(s);	re=0;	}
+		}
+	}
+	else
+	{
+		double a,b;
+		int s1=sscanf(s,"%lg+i%lg",&re,&im);
+		int s2=sscanf(s,"%lg-i%lg",&a,&b);
+		if(s1<2)
+		{
+		if(s2==2)	{	re=a;	im=-b;	}
+		else	{	re=atof(s);	im=0;	}
+		}
+	}
+	setlocale(LC_NUMERIC, "");
+	return dual(re,im);
+}
+//-----------------------------------------------------------------------------
 void DatPanel::putValue(int r, int c)
 {
 	if(!var || r<0 || c<0 || r>=ny || c>=nx || !ready)	return;
 	QString s = tab->item(r,c)->text().toLower();
 	mreal f;
+	dual g;
 	if(s=="nan")	f=NAN;
 	else if(s=="inf")	f=INFINITY;
 	else if(s=="-inf")	f=-INFINITY;
-	else	f = s.toDouble();
-	if(f!=var->v(c,r,kz))
+	else	g = mgl_str2dual(s.toLocal8Bit().constData());	//f = s.toDouble();
+	f = real(g);
+	mglDataC *cc = dynamic_cast<mglDataC*>(var);
+	if(cc)
 	{
-		if(mgl_isnan(f))	s="nan";
-		else if(mgl_isbad(f))	s=f>0?"inf":"-inf";
-		else	s.sprintf("%g", f);
-		tab->item(r,c)->setText(s);
+		if(g!=cc->a[c+nx*(r+ny*kz)])
+		{
+			if(mgl_isnan(g))	s="nan";
+			else if(mgl_isbad(g))	s="inf";
+			else if(imag(g)>0)	s.sprintf("%g+%gi",real(g),imag(g));
+			else if(imag(g)<0)	s.sprintf("%g-%gi",real(g),-imag(g));
+			else	s.sprintf("%g",real(g));
+			tab->item(r,c)->setText(s);
+		}
+		cc->a[c+nx*(r+ny*kz)] = g;
+	}
+	else
+	{
+		if(f!=var->v(c,r,kz))
+		{
+			if(mgl_isnan(f))	s="nan";
+			else if(mgl_isbad(f))	s=f>0?"inf":"-inf";
+			else	s.sprintf("%g", f);
+			tab->item(r,c)->setText(s);
+		}
+		var->set_v(f,c,r,kz);
 	}
-	var->set_v(f,c,r,kz);
 	infoDlg->refresh();
 }
 //-----------------------------------------------------------------------------
@@ -187,15 +251,15 @@ void DatPanel::save()
 	{
 		bool ok;
 		QString s = QInputDialog::getText(this, tr("UDAV - Export to PNG"), tr("Enter color scheme for picture"), QLineEdit::Normal, MGL_DEF_SCH, &ok);
-		if(ok)	var->Export(fn.toStdString().c_str(), s.toStdString().c_str());
+		if(ok)	var->Export(fn.toLocal8Bit().constData(), s.toLocal8Bit().constData());
 	}
 	else if(ext=="h5" || ext=="hdf")
 	{
 		bool ok;
-		QString s = QInputDialog::getText(this, tr("UDAV - Save to HDF"), tr("Enter data name"), QLineEdit::Normal, QString::fromStdWString(var->s), &ok);
-		if(ok)	var->SaveHDF(fn.toStdString().c_str(), s.toStdString().c_str());
+		QString s = QInputDialog::getText(this, tr("UDAV - Save to HDF"), tr("Enter data name"), QLineEdit::Normal, QString::fromWCharArray(var->s.c_str()), &ok);
+		if(ok)	var->SaveHDF(fn.toLocal8Bit().constData(), s.toLocal8Bit().constData());
 	}
-	else 	var->Save(fn.toStdString().c_str());
+	else 	var->Save(fn.toLocal8Bit().constData());
 }
 //-----------------------------------------------------------------------------
 void DatPanel::load()
@@ -209,15 +273,15 @@ void DatPanel::load()
 	{
 		bool ok;
 		QString s = QInputDialog::getText(this, tr("UDAV - Import PNG"), tr("Enter color scheme for picture"), QLineEdit::Normal, MGL_DEF_SCH, &ok);
-		if(ok)	d->Import(fn.toStdString().c_str(), s.toStdString().c_str());
+		if(ok)	d->Import(fn.toLocal8Bit().constData(), s.toLocal8Bit().constData());
 	}
 	else if(ext=="h5" || ext=="hdf")
 	{
 		bool ok;
-		QString s = QInputDialog::getText(this, tr("UDAV - Read from HDF"), tr("Enter data name"), QLineEdit::Normal, QString::fromStdWString(var->s), &ok);
-		if(ok)	d->ReadHDF(fn.toStdString().c_str(), s.toStdString().c_str());
+		QString s = QInputDialog::getText(this, tr("UDAV - Read from HDF"), tr("Enter data name"), QLineEdit::Normal, QString::fromWCharArray(var->s.c_str()), &ok);
+		if(ok)	d->ReadHDF(fn.toLocal8Bit().constData(), s.toLocal8Bit().constData());
 	}
-	else 	d->Read(fn.toStdString().c_str());
+	else 	d->Read(fn.toLocal8Bit().constData());
 	refresh();
 }
 //-----------------------------------------------------------------------------
@@ -401,8 +465,8 @@ void DatPanel::hist()
 	bool res = d->exec();
 	if(res && !v1->text().isEmpty() && !v2->text().isEmpty() && !id->text().isEmpty())
 	{
-		mglData *vv = parser.AddVar(id->text().toStdString().c_str());
-		vv->Set(mglData(true,mgl_data_hist(var, nm->value(), v1->text().toDouble(), v2->text().toDouble(),0)));
+		mglData *vv = dynamic_cast<mglData*>(parser.AddVar(id->text().toLocal8Bit().constData()));
+		if(vv)	vv->Set(mgl_data_hist(var, nm->value(), v1->text().toDouble(), v2->text().toDouble(),0));
 		updateDataItems();
 	}
 }
@@ -494,7 +558,7 @@ void DatPanel::newdat()
 	bool res = d->exec();
 	QString 	val = f1->text(), mgl;
 	int k = c->currentIndex();
-	QString self = QString::fromStdWString(var->s);
+	QString self = QString::fromWCharArray(var->s.c_str());
 	if(res)
 	{
 		if(k<0)
@@ -530,7 +594,7 @@ void DatPanel::newdat()
 	if(!mgl.isEmpty())
 	{
 		mglGraph gr;
-		parser.Execute(&gr,mgl.toStdString().c_str());
+		parser.Execute(&gr,mgl.toLocal8Bit().constData());
 		if(k>=6)	opers += mgl+"\n";
 		updateDataItems();
 	}
@@ -571,7 +635,7 @@ void DatPanel::oper()
 	bool res = d->exec();
 	QString 	val = f1->text(), mgl;
 	int k = c->currentIndex();
-	QString self = QString::fromStdWString(var->s);
+	QString self = QString::fromWCharArray(var->s.c_str());
 	if(res)
 	{
 		if(k<0)
@@ -601,7 +665,7 @@ void DatPanel::oper()
 	if(!mgl.isEmpty())
 	{
 		mglGraph gr;
-		parser.Execute(&gr,mgl.toStdString().c_str());
+		parser.Execute(&gr,mgl.toLocal8Bit().constData());
 		opers += mgl+"\n";
 		updateDataItems();
 	}
@@ -772,5 +836,5 @@ void DatPanel::toolLeft(QBoxLayout *l)
 	bb = new QToolButton(this);	l->addWidget(bb);	bb->setDefaultAction(a);
 }
 //-----------------------------------------------------------------------------
-QString DatPanel::dataName()	{	return QString::fromStdWString(var->s);	}
+QString DatPanel::dataName()	{	return QString::fromWCharArray(var->s.c_str());	}
 //-----------------------------------------------------------------------------
