@@ -131,9 +131,9 @@ StyleDialog::StyleDialog(QWidget *parent) : QDialog(parent)
 	mask = new QComboBox(p);	g->addWidget(mask, 2, 1);	fillMasks(mask);
 	angle = new QComboBox(p);	g->addWidget(angle, 3, 1);
 	angle->addItem(tr("none"));	
-	angle->addItem(QString::fromWCharArray(L"+45°"));
-	angle->addItem(QString::fromWCharArray(L"-45°"));	
-	angle->addItem(QString::fromWCharArray(L"90°"));
+	angle->addItem(QString::fromWCharArray(L"+45\xb0"));
+	angle->addItem(QString::fromWCharArray(L"-45\xb0"));	
+	angle->addItem(QString::fromWCharArray(L"90\xb0"));	// \xb0 <-> °
 	msize = new QSlider(p);		g->addWidget(msize, 4, 1);
 	msize->setRange(1, 9);		msize->setValue(1);
 	msize->setTickPosition(QSlider::TicksBothSides);
@@ -451,7 +451,7 @@ void StyleDialog::updatePic()
 			else		result += col[i-1];
 		}
 		i = width->value();		if(i>1)	result += char('0'+i);
-		gr.Plot(x,y,result.toStdString().c_str());
+		gr.Plot(x,y,result.toLocal8Bit().constData());
 		break;
 	case 1: // color sceheme
 	case 3: // manual mask
@@ -492,7 +492,7 @@ void StyleDialog::updatePic()
 		
 		i = axial->currentIndex();
 		if(i>0)	result = result+':'+char('x'+i-1);
-		gr.Surf(a,result.toStdString().c_str());
+		gr.Surf(a,result.toLocal8Bit().constData());
 		break;
 	case 2: // text style
 		if(font_sch->isChecked())	for(j=0;j<7;j++)
@@ -518,7 +518,7 @@ void StyleDialog::updatePic()
 		if(rbL->isChecked())	result += 'L';
 		if(rbC->isChecked())	result += 'C';
 		if(rbR->isChecked())	result += 'R';
-		gr.Puts(mglPoint(0,-0.5),"Font test",result.toStdString().c_str(),-10);
+		gr.Puts(mglPoint(0,-0.5),"Font test",result.toLocal8Bit().constData(),-10);
 		break;
 	}
 	result = "'" + result + "'";
