@@ -53,9 +53,10 @@ bool editPosBottom = false;
 bool mglAutoSave = false;
 bool mglHighlight = true;
 bool mglDotsRefr = true;
-bool mglAutoPure = true;
+// bool mglAutoPure = true;
 bool mglCompleter = true;
 bool loadInNewWnd = false;
+bool mglWheelZoom = false;
 QString pathHelp;
 extern mglParse parser;
 extern QColor mglColorScheme[10];
@@ -132,6 +133,7 @@ int main(int argc, char **argv)
 		mw->load(codec->toUnicode(argv[1]), true);
 	}
 	mw->show();
+	mw->edit->edit->setFocus();
 	a.connect(&a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()));
 	if(showHint)	udavShowHint(mw);
 	return a.exec();
@@ -161,7 +163,7 @@ MainWindow::MainWindow(QWidget *wp) : QMainWindow(wp)
 	rtab->setMovable(true);	rtab->setTabPosition(QTabWidget::South);
 
 	messWnd = new QDockWidget(tr("Messages and warnings"),this);
-	mess = new TextEdit(this);	messWnd->setWidget(mess);
+	mess = new QTextEdit(this);	messWnd->setWidget(mess);
 	messWnd->setAllowedAreas(Qt::TopDockWidgetArea | Qt::BottomDockWidgetArea);
 	addDockWidget(Qt::BottomDockWidgetArea, messWnd);
 	messWnd->resize(size().width(), 0);	new MessSyntax(mess);
@@ -217,6 +219,7 @@ MainWindow::MainWindow(QWidget *wp) : QMainWindow(wp)
 	edit = new TextPanel(this);	edit->graph = graph;
 	graph->textMGL = edit->edit;
 	connect(graph->mgl,SIGNAL(showWarn(QString)),mess,SLOT(setText(QString)));
+	connect(graph->mgl,SIGNAL(showWarn(QString)),edit->edit,SLOT(setErrMessage(QString)));
 	connect(graph,SIGNAL(clearWarn()),mess,SLOT(clear()));
 	ltab->addTab(edit,QPixmap(":/png/text-plain.png"),tr("Script"));
 
@@ -421,9 +424,11 @@ void MainWindow::editPosChanged()
 	for(i=0;i<n;i++)	if(dlm.contains(text[i]))	break;
 	text.truncate(i);
 
-	const char *desc = parser.CmdDesc(text.toStdString().c_str());
-	const char *form = parser.CmdFormat(text.toStdString().c_str());
-	if(form)	setStatus(QString::fromLocal8Bit(desc)+": "+QString::fromLocal8Bit(form));
+	QByteArray qxtext = text.toLatin1();
+	const char *ctext = qxtext.constData();
+	const char *desc = parser.CmdDesc(ctext);
+	const char *form = parser.CmdFormat(ctext);
+	if(form)	setStatus(QString(desc)+": "+QString(form));
 	else	setStatus(tr("Not recognized"));
 }
 //-----------------------------------------------------------------------------
@@ -471,12 +476,13 @@ void MainWindow::writeSettings()
 	settings.setValue("/autoSave",  mglAutoSave);
 	settings.setValue("/highlight",  mglHighlight);
 	settings.setValue("/dotsRefresh", mglDotsRefr);
-	settings.setValue("/autoPure",  mglAutoPure);
+// 	settings.setValue("/autoPure",  mglAutoPure);
 	settings.setValue("/editAtTop", editPosBottom);
 	settings.setValue("/fontFamily", defFontFamily);
 	settings.setValue("/fontSize", defFontSize);
 	settings.setValue("/loadInNewWnd", loadInNewWnd);
 	settings.setValue("/completer",  mglCompleter);
+	settings.setValue("/wheelZoom",  mglWheelZoom);
 	settings.endGroup();
 }
 //-----------------------------------------------------------------------------
@@ -506,18 +512,20 @@ void MainWindow::readSettings()
 	mglColorScheme[7] = QColor(settings.value("/colFCKeyword","#007F7F").toString());
 	mglColorScheme[8] = QColor(settings.value("/colReserved", "#0000FF").toString());
 	mglColorScheme[9] = QColor(settings.value("/colCurrLine", "#FFFFCC").toString());
-	mglAutoSave = settings.value("/autoSave",  false).toBool();
-	mglHighlight = settings.value("/highlight",  true).toBool();
-	mglAutoPure = settings.value("/autoPure",  true).toBool();
-	mglAutoExecute = settings.value("/autoExec",  true).toBool();
+	mglAutoSave = settings.value("/autoSave", false).toBool();
+	mglHighlight = settings.value("/highlight", true).toBool();
+// 	mglAutoPure = settings.value("/autoPure", true).toBool();
+	mglAutoExecute = settings.value("/autoExec", true).toBool();
 	editPosBottom = settings.value("/editAtTop", false).toBool();
-	mglCompleter = settings.value("/completer",  true).toBool();
+	mglCompleter = settings.value("/completer", true).toBool();
+	mglWheelZoom = settings.value("/wheelZoom", false).toBool();
 	loadInNewWnd = settings.value("/loadInNewWnd", false).toBool();
 	mglDotsRefr = settings.value("/dotsRefresh", true).toBool();
 	defFontFamily = settings.value("/fontFamily", "Georgia").toString();
 	defFontSize = settings.value("/fontSize", 10).toInt();
 	edit->setEditorFont();	setEditPos(editPosBottom);
 	graph->setMGLFont(pathFont);
+	graph->mgl->enableWheel = mglWheelZoom;
 
 	defWidth = settings.value("/defWidth", 640).toInt();
 	defHeight = settings.value("/defHeight", 480).toInt();
@@ -534,7 +542,7 @@ void MainWindow::setStatus(const QString &txt)
 void MainWindow::setCurrentFile(const QString &fileName)
 {
 	filename = fileName;
-	mgl_set_plotid(graph->mgl->getGraph(), fileName.toStdString().c_str());
+	mgl_set_plotid(graph->mgl->getGraph(), fileName.toLocal8Bit().constData());
 	edit->setModified(false);
 	if(filename.isEmpty())
 		setWindowTitle(tr("untitled - UDAV"));
