@@ -141,12 +141,12 @@ int main(int argc, char *argv[])
 		for(size_t i=0;i<var.size();i++)
 		{
 			gr.NewFrame();
-			printf("frame %ld for $0 = \"%ls\"\n",i,var[i].c_str());
+			printf("frame %zu for $0 = \"%ls\"\n",i,var[i].c_str());
 			p.AddParam(0,var[i].c_str());
 			p.Execute(&gr,str.c_str());
 			if(gr.Message()[0])	printf("%s\n",gr.Message());
 			gr.EndFrame();
-			snprintf(buf,2048,"%s-%ld",oname,i);	buf[2047]=0;
+			snprintf(buf,2048,"%s-%zu",oname,i);	buf[2047]=0;
 			if(!gif)	gr.WriteFrame(buf);
 		}
 		if(gif)	gr.CloseGIF();
