@@ -22,13 +22,10 @@
 
 #include "mgl2/mgl.h"
 #include "mgl2/qt.h"
+#include "mgl2/parser.h"
 //-----------------------------------------------------------------------------
 std::wstring str, opt;
 mglParse p(true);
-void mgl_error_print(const char *Message, void *par);
-void mgl_ask_fltk(const wchar_t *quest, wchar_t *res);
-void mgl_ask_qt(const wchar_t *quest, wchar_t *res);
-void mgl_ask_gets(const wchar_t *quest, wchar_t *res);
 //-----------------------------------------------------------------------------
 int show(mglGraph *gr)
 {
