@@ -1,6 +1,6 @@
 /***************************************************************************
  * fltk.cpp is part of Math Graphic Library
- * Copyright (C) 2007-2014 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
+ * Copyright (C) 2007-2016 Alexey Balakin <mathgl.abalakin@gmail.ru>       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -175,6 +175,14 @@ void Fl_MathGL::update()
 		const char *buf = mgl_get_mess(gr);
 		if(*buf)	fl_message("%s",buf);
 	}
+	else if(mgl_get_num_frame(gr)>0)
+	{
+		mgl_set_alpha(gr,flag&1);	mgl_set_light(gr,flag&2);
+		if(tet_val)	tet = tet_val->value();
+		if(phi_val)	phi = phi_val->value();
+		mgl_zoom(gr,x1,y1,x2,y2);	mgl_view(gr,-phi,-tet,0);
+		mgl_get_frame(gr,0);
+	}
 	if(mgl_get_width(gr)!=w() || mgl_get_height(gr)!=h())
 		size(mgl_get_width(gr), mgl_get_height(gr));
 	gr->AskStop(false);	redraw();
@@ -349,8 +357,8 @@ void Fl_MGLView::exec_pause()
 	}
 	if(mutex)
 	{
-		if(pauseC)	pthread_mutex_lock(mutex);
-		else	pthread_mutex_unlock(mutex);
+		pthread_mutex_trylock(mutex);
+		if(!pauseC)	pthread_mutex_unlock(mutex);
 	}
 #endif
 }
@@ -764,6 +772,11 @@ HMGL MGL_EXPORT mgl_create_graph_fltk(int (*draw)(HMGL gr, void *p), const char
 	g->Window(0,0,draw,title,par,load);
 	return g;
 }
+void* mgl_fltk_widget(HMGL gr)
+{
+	mglCanvasFL *g = dynamic_cast<mglCanvasFL *>(gr);
+	return g?g->mgl:NULL;
+}
 int MGL_EXPORT mgl_fltk_run()		{	return Fl::run();	}
 //-----------------------------------------------------------------------------
 uintptr_t MGL_EXPORT mgl_create_graph_fltk_(const char *title, int l)
