@@ -366,7 +366,7 @@ void QMathGL::drawPrim()
 	{
 		mgl_set_obj_id(gr,i+MGL_MAX_LINES);
 		QString tst = primitives.section('\n',i,i);
-		pr.Parse(&gg,primitives.section('\n',i,i).toStdString().c_str(),i+MGL_MAX_LINES);
+		pr.Parse(&gg,primitives.section('\n',i,i).toLocal8Bit().constData(),i+MGL_MAX_LINES);
 	}
 	gg.SetRanges(ox1,ox2);	gg.Pop();	setlocale(LC_NUMERIC, "");
 }
@@ -678,7 +678,7 @@ void QMathGL::exportGIF(QString fname)
 	if(fname.isEmpty())	QMessageBox::critical(this, appName, tr("No filename."),QMessageBox::Ok,0,0);
 	else
 #if MGL_HAVE_GIF
-		mgl_write_gif(gr,setExtension(fname,"png").toStdString().c_str(), appName.toStdString().c_str());
+		mgl_write_gif(gr,setExtension(fname,"png").toLocal8Bit().constData(), appName.toLocal8Bit().constData());
 #else
 		img.save(setExtension(fname,"gif"));
 #endif
@@ -690,7 +690,7 @@ void QMathGL::exportPNG(QString fname)
 	if(fname.isEmpty())	QMessageBox::critical(this, appName, tr("No filename."),QMessageBox::Ok,0,0);
 	else
 #if MGL_HAVE_PNG
-		mgl_write_png(gr,setExtension(fname,"png").toStdString().c_str(), appName.toStdString().c_str());
+		mgl_write_png(gr,setExtension(fname,"png").toLocal8Bit().constData(), appName.toLocal8Bit().constData());
 #else
 		img.save(setExtension(fname,"png"));
 #endif
@@ -702,7 +702,7 @@ void QMathGL::exportPNGs(QString fname)
 	if(fname.isEmpty())	QMessageBox::critical(this, appName, tr("No filename."),QMessageBox::Ok,0,0);
 	else
 #if MGL_HAVE_PNG
-		mgl_write_png_solid(gr,setExtension(fname,"png").toStdString().c_str(), appName.toStdString().c_str());
+		mgl_write_png_solid(gr,setExtension(fname,"png").toLocal8Bit().constData(), appName.toLocal8Bit().constData());
 #else
 		img.save(setExtension(fname,"png"));
 #endif
@@ -714,7 +714,7 @@ void QMathGL::exportJPG(QString fname)
 	if(fname.isEmpty())	QMessageBox::critical(this, appName, tr("No filename."),QMessageBox::Ok,0,0);
 	else
 #if MGL_HAVE_JPEG
-		mgl_write_jpg(gr,setExtension(fname,"jpg").toStdString().c_str(), appName.toStdString().c_str());
+		mgl_write_jpg(gr,setExtension(fname,"jpg").toLocal8Bit().constData(), appName.toLocal8Bit().constData());
 #else
 		img.save(setExtension(fname,"jpg"));
 #endif
@@ -727,7 +727,7 @@ void QMathGL::exportBPS(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_bps(gr,setExtension(fname,"eps").toStdString().c_str(), appName.toStdString().c_str());
+		mgl_write_bps(gr,setExtension(fname,"eps").toLocal8Bit().constData(), appName.toLocal8Bit().constData());
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -739,7 +739,7 @@ void QMathGL::exportEPS(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_eps(gr,setExtension(fname,"eps").toStdString().c_str(), appName.toStdString().c_str());
+		mgl_write_eps(gr,setExtension(fname,"eps").toLocal8Bit().constData(), appName.toLocal8Bit().constData());
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -751,7 +751,7 @@ void QMathGL::exportSVG(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_svg(gr,setExtension(fname,"svg").toStdString().c_str(), appName.toStdString().c_str());
+		mgl_write_svg(gr,setExtension(fname,"svg").toLocal8Bit().constData(), appName.toLocal8Bit().constData());
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -763,7 +763,7 @@ void QMathGL::exportXYZ(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_xyz(gr,setExtension(fname,"xyz").toStdString().c_str(), appName.toStdString().c_str());
+		mgl_write_xyz(gr,setExtension(fname,"xyz").toLocal8Bit().constData(), appName.toLocal8Bit().constData());
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -775,7 +775,7 @@ void QMathGL::exportTEX(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_tex(gr,setExtension(fname,"tex").toStdString().c_str(), appName.toStdString().c_str());
+		mgl_write_tex(gr,setExtension(fname,"tex").toLocal8Bit().constData(), appName.toLocal8Bit().constData());
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -787,7 +787,7 @@ void QMathGL::exportOFF(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_off(gr,setExtension(fname,"off").toStdString().c_str(), appName.toStdString().c_str(),0);
+		mgl_write_off(gr,setExtension(fname,"off").toLocal8Bit().constData(), appName.toLocal8Bit().constData(),0);
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -799,7 +799,7 @@ void QMathGL::exportOBJ(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_obj(gr,setExtension(fname,"obj").toStdString().c_str(), appName.toStdString().c_str(),1);
+		mgl_write_obj(gr,setExtension(fname,"obj").toLocal8Bit().constData(), appName.toLocal8Bit().constData(),1);
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -811,7 +811,7 @@ void QMathGL::exportSTL(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_stl(gr,setExtension(fname,"stl").toStdString().c_str(), appName.toStdString().c_str());
+		mgl_write_stl(gr,setExtension(fname,"stl").toLocal8Bit().constData(), appName.toLocal8Bit().constData());
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -823,7 +823,7 @@ void QMathGL::exportSTL(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_x3d(gr,setExtension(fname,"x3d").toStdString().c_str(), appName.toStdString().c_str());
+		mgl_write_x3d(gr,setExtension(fname,"x3d").toLocal8Bit().constData(), appName.toLocal8Bit().constData());
 		setlocale(LC_NUMERIC, "");
 	}
 }*/
@@ -835,7 +835,7 @@ void QMathGL::exportTGA(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_tga(gr,setExtension(fname,"tga").toStdString().c_str(), appName.toStdString().c_str());
+		mgl_write_tga(gr,setExtension(fname,"tga").toLocal8Bit().constData(), appName.toLocal8Bit().constData());
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -847,7 +847,7 @@ void QMathGL::exportPRC(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_write_prc(gr,setExtension(fname,"prc").toStdString().c_str(), appName.toStdString().c_str(),1);
+		mgl_write_prc(gr,setExtension(fname,"prc").toLocal8Bit().constData(), appName.toLocal8Bit().constData(),1);
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -859,7 +859,7 @@ void QMathGL::exportMGLD(QString fname)
 	else
 	{
 		setlocale(LC_NUMERIC, "C");
-		mgl_export_mgld(gr,setExtension(fname,"mgld").toStdString().c_str(), appName.toStdString().c_str());
+		mgl_export_mgld(gr,setExtension(fname,"mgld").toLocal8Bit().constData(), appName.toLocal8Bit().constData());
 		setlocale(LC_NUMERIC, "");
 	}
 }
@@ -891,7 +891,7 @@ void QMathGL::copyClickCoor()
 //-----------------------------------------------------------------------------
 void QMathGL::setMGLFont(QString path)
 {	if(path.isEmpty())	mgl_restore_font(gr);
-	else 	mgl_load_font(gr,path.toStdString().c_str(),0);	}
+	else 	mgl_load_font(gr,path.toLocal8Bit().constData(),0);	}
 //-----------------------------------------------------------------------------
 void QMathGL::setSize(int w, int h)
 {
